/*
 *  $Id: dal.h,v 1.31 2000/11/14 17:43:20 nick Exp $
 *
 *  $Log: dal.h,v $
 *  Revision 1.31  2000/11/14 17:43:20  nick
 *  Add IS NULL support
 *
 *  Revision 1.30  2000/10/12 17:37:57  nick
 *  reapply previous changes
 *
 *  Revision 1.29  2000/10/12 14:28:53  nick
 *  adjusted by jsc so that checked in dal layer compiles
 *
 *  Revision 1.28  2000/10/11 21:20:59  nick
 *  First result set caching code
 *
 *  Revision 1.27  2000/09/19 12:47:44  nick
 *  Add OOO_GD define
 *
 *  Revision 1.26  2000/07/10 15:16:31  nick
 *  Add support for 4 part names
 *
 *  Revision 1.25  2000/06/19 08:37:20  nick
 *  Add 1.4 support code
 *
 *  Revision 1.24  2000/06/05 08:18:32  nick
 *  Add Passwthrough support
 *
 *  Revision 1.23  2000/02/15 12:07:03  nick
 *  Add procedure support
 *
 *  Revision 1.22  2000/01/12 13:51:14  nick
 *  Add DSN to DALOpen
 *
 *  Revision 1.21  1999/11/08 17:51:00  nick
 *  Add support for sqi get get UID/PWD from the connect
 *
 *  Revision 1.20  1999/10/15 15:15:57  nick
 *  Add col_used flag
 *
 *  Revision 1.19  1999/09/22 11:32:52  nick
 *  Add HELPER_GET_METADATA_ID and gti_column_size
 *
 *  Revision 1.18  1999/06/30 13:29:38  nick
 *  Add some extra fields to cope with expansion
 *  Add support for auto incrementing fields
 *
 *  Revision 1.17  1999/05/04 12:15:17  nick
 *  Add DAL_INVALID to signal no usable key
 *
 *  Revision 1.16  1999/04/16 08:17:28  nick
 *  Support for VARBOOK
 *  the DAL driver can query a driver to check if it supports numerics
 *  Extra args to StartQuery
 *
 *  Revision 1.15  1999/02/26 11:45:31  nick
 *  Add weighting call
 *
 *  Revision 1.14  1999/02/22 10:49:41  nick
 *  Add modified filed to the QueryQualifier struct for use with update
 *
 *  Revision 1.13  1999/02/16 11:33:36  nick
 *  Add DALTableDefinition and extra flags to DALColumnDefinition
 *  Add extra args to DALCreateTable
 *
 *  Revision 1.12  1999/01/25 15:59:43  nick
 *  Add special DAL_ROWLONG column type
 *  Alter args for DALConnect
 *
 *  Revision 1.11  1999/01/04 13:02:29  nick
 *  Add extra arg to DALGetColumnInfo
 *
 *  Revision 1.10  1998/12/15 10:29:40  nick
 *  ScoreQuery now takes a array of pointers
 *
 *  Revision 1.9  1998/12/03 17:21:15  nick
 *  Add three extra callback routines
 *
 *  Revision 1.8  1998/11/20 13:19:22  nick
 *  Add DALScoreQuery
 *
 *  Revision 1.7  1998/11/18 11:47:08  nick
 *  add extra columndatatype fields to handle the gettypeinfo call
 *
 *  Revision 1.6  1998/11/10 13:34:51  nick
 *  Add VIEW support
 *
 *  Revision 1.5  1998/10/20 09:27:11  nick
 *  Added extra fields and args to support concurrency
 *
 *  Revision 1.4  1998/09/21 14:18:52  nick
 *  Add long var helper structures
 *
 *  Revision 1.3  1998/08/18 15:26:42  nick
 *  Add column_id to the index structure
 *
 *  Revision 1.2  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.1  1998/05/08 08:54:55  nick
 *  Initial revision
 *
 *
 */

#ifndef _DAL_H
#define _DAL_H

#include "connfunc.h"

#define DAL_SUCCESS                 0
#define DAL_SUCCESS_WITH_INFO       1
#define DAL_NO_DATA                 2
#define DAL_ERROR                   3
#define DAL_TABLE_NOT_FOUND         4
#define DAL_LOCKED                  5

#define DAL_SCHEMA_LENGTH           128
#define DAL_CATALOG_LENGTH          128
#define DAL_NAME_LENGTH             128
#define DAL_COLUMN_LENGTH           128

#define DAL_INDEX_UNIQUE            0
#define DAL_INDEX_NONUNIQUE         1
#define DAL_INDEX_DESC              0
#define DAL_INDEX_ASC               1

#define SQI_INFO_ODBC_VERSION       0

/*
 * special column types
 */
#define DAL_ROWLONG                 -1
#define DAL_ROWVARBOOK              -2
#define DAL_VARBOOKLENGTH           32
#define DAL_INVALID                 -3

typedef void * DALHANDLE;
typedef void * DALITERATOR;
typedef void * DALPARSEHANDLE;
typedef void (*HELPERFUNC)(int,void*);

/* callback types */

#define HELPER_ERROR                0x0001
#define HELPER_MALLOC               0x0002
#define HELPER_BUFLEN               0x0003
#define HELPER_BUFFER               0x0004
#define HELPER_COMPILE_LIKE         0x0005
#define HELPER_CHECK_EXPR           0x0006
#define HELPER_GET_EXTEND_INFO      0x0007
#define HELPER_BUFREW               0x0008
#define HELPER_GET_METADATA_ID      0x0009
#define HELPER_GET_UID_PASS         0x000A

/* GetInfo Types */

#define DAL_INFO_GET_QUAL_MASK      0x0001      /* types of qualifiers that can be passed */
#define DAL_INFO_QUAL_TWO_COL       0x0001
#define DAL_INFO_QUAL_LIKE          0x0002
#define DAL_INFO_QUAL_NULL          0x0004

#define DAL_INFO_GET_WEIGHTING      0x0002
#define DAL_INFO_HAS_NUMERICS       0x0003
#define DAL_INFO_OOO_GD             0x0004

typedef struct
{
    SQLHANDLE       handle;
    char            *driver_name;
    int             native;
    char            *sqlstate;
    char            *txt;
} HELPER_ERROR_STR;

typedef struct
{
    void            *handle;
    int             length;
    void            *ptr;
} HELPER_MALLOC_STR;

typedef struct 
{
    void            *value;
    int             length;
} HELPER_BUFLEN_STR;

typedef struct 
{
    void            *value;
} HELPER_BUFREW_STR;

typedef struct 
{
    void            *value;
    void            *buffer;
    int             max;
    int             length;
    int             ret;
} HELPER_BUFFER_STR;

typedef struct 
{
    void            *like;
    void            *memhandle;
    int             ret;
} HELPER_COMPILE_LIKE_STR;

typedef struct
{
    int             *result;
    int             ret;
    void            *expr;
    void            *memhandle;
} HELPER_CHECK_EXPR_STR;

typedef struct 
{
    void            *dt;
    int             type;
    int             ret;
} HELPER_GET_EXTEND_INFO_STR;

typedef struct
{
    SQLHANDLE       handle;
    int             ret;
} HELPER_GET_METADATA_STR;

typedef struct 
{
    SQLHANDLE       dbc;
    char            uid[ 128 ];
    char            pwd[ 128 ];
} HELPER_GET_UID_PASS_STR;


typedef struct ColumnDataType
{
    int                 consise_type;
    int                 interval_code;
    int                 interval_leading_precision;
    int                 display_size;
    int                 precision_scale;
    char                literal_prefix[ 64 ], literal_suffix[ 64 ];
    char                local_type_name[ 64 ], type_name[ 64 ];
    int                 nullable;
    int                 radix;
    int                 octet_length;
    int                 precision;
    int                 scale;
    int                 auto_unique;
    int                 type;
    int                 searchable;
    int                 case_sensitive;
    int                 unsig;
    int                 updateable;
    int                 column_size;
    int                 gti_column_size;
    char                create_params[ 64 ];
    int                 minimum_scale;
    int                 maximum_scale;
    void                *reserved1;
    void                *reserved2;
} ColumnDataType;

typedef struct 
{
    char                catalog[ DAL_CATALOG_LENGTH ];
    char                schema[ DAL_SCHEMA_LENGTH ];
    char                name[ DAL_NAME_LENGTH ];
    int                 column_count;
    int                 pk_column_count;
    int                 lower_mux;
    int                 concur_length;
    void                *reserved1;
    void                *reserved2;
    int                 is_relational;
} SQITABLEINFO;

typedef struct
{
    char                catalog[ DAL_CATALOG_LENGTH ];
    char                schema[ DAL_SCHEMA_LENGTH ];
    char                name[ DAL_NAME_LENGTH ];
    int                 column_count;
    int                 pk_column_count;
    int                 lower_mux;
    int                 concur_length;
    void                *reserved1;
    void                *reserved2;
    int                 is_relational;
    char                link[ DAL_CATALOG_LENGTH ];
} DALTABLEINFO;

typedef struct
{
    char                catalog[ DAL_CATALOG_LENGTH ];
    char                schema[ DAL_SCHEMA_LENGTH ];
    char                name[ DAL_NAME_LENGTH ];
    char                column[ DAL_COLUMN_LENGTH ];
    ColumnDataType      data_type_info;
    int                 column_id;
    int                 lower_mux;
    int                 col_used;
    void                *reserved2;
} DALCOLUMNINFO;

typedef struct 
{
    char                catalog[ DAL_CATALOG_LENGTH ];
    char                schema[ DAL_SCHEMA_LENGTH ];
    char                proc[ DAL_NAME_LENGTH ];
    int                 param_count;
    int                 lower_mux;
} DALPROCINFO;

typedef struct
{
    char                catalog[ DAL_CATALOG_LENGTH ];
    char                schema[ DAL_SCHEMA_LENGTH ];
    char                proc[ DAL_NAME_LENGTH ];
    char                arg[ DAL_COLUMN_LENGTH ];
    int                 column_id;
    int                 column_type;
    int                 data_type;
    int                 column_size;
    int                 buffer_length;
    int                 decimal_digits;
    int                 datetime_sub;
    int                 odbc3_data_type;
    int                 octet_length;
    char                type_name[ 128 ];
    int                 nullable;
    int                 radix;
    int                 lower_mux;
    SQLINTEGER          strlen_or_ind;
} DALARGINFO;

typedef struct QueryQualifier
{
    DALCOLUMNINFO       *column_info;           /* in the table in question */
    int                 column_id;
    DALCOLUMNINFO       *column_info2;
    int                 column_id2;
    int                 operation;              /* = != < etc */
    void                *expr;                  /* extracted by the tree walker */
    void                *value;                 /* generated from the expr */
    void                *column;
    int                 success_count;
    int                 modified;               /* signal that a update is changing this */
    void                *reserved1;
    void                *reserved2;
} QueryQualifier;

typedef struct ColumnDefinitions
{
    char                column_name[ DAL_COLUMN_LENGTH ];
    int                 base_type;
    int                 length;
    int                 precision;
    int                 scale;
    void                *default_value;
    int                 unique;
    int                 non_null;
    int                 auto_start;
    int                 auto_inc;
    char                ref_table_name[ DAL_NAME_LENGTH ];
    char                ref_column_name[ DAL_NAME_LENGTH ];
    char                *column_string;
    int                 key_number;
} DALColumnDefinitions;

typedef struct TableDefinitions
{
    int                     definition_type;
    int                     column_list_1_count;
    DALColumnDefinitions    *column_list_1;
    int                     column_list_2_count;
    DALColumnDefinitions    *column_list_2;
    char                    *sql;
    char                    *file_name;
    int                     key_number;
} DALTableDefinitions;

typedef struct IndexDefinitions
{
    char                column_name[ DAL_COLUMN_LENGTH ];
    int                 column_id;
    int                 direction;
    int                 key_number;
    void                *reserved2;
} DALIndexDefinition;

typedef struct PrivDefinition
{
    int                    priv;
    char                *user;    
    char                *column_name;
    int                    column_id;
    int                    with_grant_option;
    int                    cascade_or_restrict;
    void                *reserved1;
    void                *reserved2;
} DALPrivDefinition;

int DALOpen( SQLHANDLE, DALHANDLE *, HELPERFUNC hf, char *dsn );
int DALClose( DALHANDLE );
int DALConnect( DALHANDLE, struct con_struct *, void * );
int DALDisconnect( DALHANDLE );
int DALGetInfo( DALITERATOR, DALTABLEINFO *, int info, void * data );
int DALSetInfo( DALHANDLE, int info, void * data );
int DALGetTableInfo( SQLHANDLE, DALHANDLE, char *, char *, char *, char *, DALTABLEINFO * );
int DALGetColumnInfo( SQLHANDLE, DALHANDLE, char *, char *, char *, int, 
        DALCOLUMNINFO *, DALTABLEINFO * );
int DALGetProcInfo( SQLHANDLE, DALHANDLE, char *, char *, char *, char *, DALPROCINFO * );
int DALGetArgInfo( SQLHANDLE, DALHANDLE, char *, char *, char *, 
      char *, int, DALARGINFO *, DALPROCINFO * );
DALITERATOR DALOpenIterator( SQLHANDLE, DALHANDLE );
void DALCloseIterator( DALITERATOR );
void DALResetIterator( DALITERATOR );
int DALFetch( DALITERATOR );
int DALTables( DALITERATOR, char *, int, char *, int, char *, int, char *, int );
int DALColumns( DALITERATOR, char *, int, char *, int, char *, int, char *, int );
int DALColumnPrivileges( DALITERATOR, char *, int, char *, int, char *, int, char *, int );
int DALPrimaryKeys( DALITERATOR, char *, int, char *, int, char *, int );
int DALSpecialColumns( DALITERATOR, int, char *, int, char *, int, char *, int, int, int );
int DALStatistics( DALITERATOR, char *, int, char *, int, char *, int, int, int );
int DALForeignKeys( DALITERATOR, char *, int, char *, int, char *, int, char *, int,
    char *, int, char *, int );
int DALTablePrivileges( DALITERATOR, char *, int, char *, int, char *, int );
int DALGetData( DALITERATOR vdi, int, int, void *, int, long * );
int DALStartQuery( DALITERATOR, DALTABLEINFO *, DALCOLUMNINFO *, int *, int, QueryQualifier ** );
int DALGetTypeInfo( DALITERATOR, int );
int DALCreateTable( DALITERATOR, DALTABLEINFO *, int, DALColumnDefinitions *,
        int, DALColumnDefinitions *, int, DALTableDefinitions * );
int DALDropTable( DALITERATOR, DALTABLEINFO * );
int DALDropIndex( DALITERATOR, DALTABLEINFO * );
int DALCreateIndex( DALITERATOR, DALTABLEINFO *, DALTABLEINFO *, int, int, 
    DALIndexDefinition * );
int DALTablePKInfo( SQLHANDLE, DALITERATOR, char *, char *, char *, int, DALIndexDefinition *, 
        DALTABLEINFO *, DALCOLUMNINFO * );
int DALFetchRow( DALITERATOR, DALTABLEINFO *, int, QueryQualifier *, int lock_mode, 
        void *concur_buffer );
int DALInsert( DALITERATOR, DALTABLEINFO *, int, QueryQualifier *, int * change_count );
int DALDeleteRow( DALITERATOR, DALTABLEINFO *, int, QueryQualifier *, int * change_count );
int DALUpdateRow( DALITERATOR, DALTABLEINFO *, int, QueryQualifier *,
    int, QueryQualifier *, int * change_count );
int DALStartTransaction( SQLHANDLE, DALHANDLE );
int DALEndTransaction( SQLHANDLE, DALHANDLE, int mode );
int DALGrant( DALITERATOR, DALTABLEINFO *, DALPrivDefinition * );
int DALRevoke( DALITERATOR, DALTABLEINFO *, DALPrivDefinition * );
int DALDropColumn( DALITERATOR, DALTABLEINFO *, char *, int );
int DALAddColumn( DALITERATOR, DALTABLEINFO *, int, DALColumnDefinitions * );
int DALScoreQuery( DALITERATOR vdi, DALTABLEINFO *dti, 
    int qualifier_count, QueryQualifier **qualifiers, int *metric );
int DALAddConstraint( DALITERATOR, DALTABLEINFO *, 
        int, DALColumnDefinitions *, int, DALTableDefinitions * );
int DALProcedures( DALITERATOR, char *, int, char *, int, char *, int );
int DALProcedureColumns( DALITERATOR, char *, int, char *, int, char *, int, char *, int );
int DALCallProcedure( DALITERATOR, DALPROCINFO *, DALARGINFO *, int, QueryQualifier *, int, QueryQualifier *i, int * );
int DALCreateUser( DALITERATOR, char *, char *, char * );
int DALDropUser( DALITERATOR, char *, char * );

/*
 * new pass through funcs
 */

int DALExecute( SQLHANDLE sh, DALHANDLE dh, DALTABLEINFO * dti );
int DALPrepare( DALITERATOR, DALTABLEINFO*, DALPARSEHANDLE, int, int *, void *, int ); 

/*
 * view interface
 */

int DALSaveView( void *stmt, void *cv );
int DALRemoveView( void *stmt, void *dv );

#endif
