/*
 *  $Id: execnodes.h,v 1.26 2000/10/11 21:20:38 nick Exp $
 *
 *  $Log: execnodes.h,v $
 *  Revision 1.26  2000/10/11 21:20:38  nick
 *  Add flag to indicate the presence of a join
 *
 *  Revision 1.25  2000/09/19 12:48:12  nick
 *  Add out_of_order_gd
 *
 *  Revision 1.24  2000/07/24 09:23:28  nick
 *  Add dead_end_query flag
 *
 *  Revision 1.23  2000/07/10 15:17:03  nick
 *  Add Create/Drop user
 *
 *  Revision 1.22  2000/06/19 08:37:48  nick
 *   Add 1.4 support code
 *
 *  Revision 1.21  2000/06/12 10:40:29  nick
 *  Add support for subselects in updates
 *
 *  Revision 1.20  2000/06/06 12:04:20  nick
 *  Add numerics_checked and qual_checked
 *
 *  Revision 1.19  2000/06/05 08:18:47  nick
 *  Add passthrough support
 *
 *  Revision 1.18  2000/03/02 17:27:07  nick
 *  Fix typo
 *
 *  Revision 1.17  2000/02/28 13:21:11  nick
 *  Add extra field
 *
 *  Revision 1.16  2000/02/28 10:04:15  nick
 *  Add fix to groupby
 *
 *  Revision 1.15  2000/02/15 12:07:47  nick
 *  Add Procedure support
 *
 *  Revision 1.14  2000/01/31 13:28:14  nick
 *  Add column info to Alter Table structure
 *
 *  Revision 1.13  1999/06/30 13:30:19  nick
 *  Add support for auto increments
 *  Add some helper structures for groupbys
 *
 *  Revision 1.12  1999/04/16 08:18:46  nick
 *  row_long_binary added
 *
 *  Revision 1.11  1999/02/26 11:45:54  nick
 *  Add weight to table index info
 *
 *  Revision 1.10  1999/02/16 11:35:21  nick
 *  Add support for COMMIT and ROLLBACK and the extra table creation constraints
 *
 *  Revision 1.9  1999/01/25 16:01:14  nick
 *  Add support for special columns ie ROWLONG
 *  Add list of tables that supply values to this correlated query
 *
 *  Revision 1.8  1999/01/04 13:05:47  nick
 *  Add support for expressions in groupby's
 *
 *  Revision 1.7  1998/12/15 10:30:31  nick
 *  Add list of parameterised like predicates
 *
 *  Revision 1.6  1998/12/03 17:21:56  nick
 *  Add field private_sub_queries to statement handle
 *
 *  Revision 1.5  1998/11/10 13:35:14  nick
 *  Add VIEW support
 *
 *  Revision 1.4  1998/08/18 15:28:19  nick
 *  checkpoint after joins
 *
 *  Revision 1.3  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.2  1998/07/13 09:40:28  nick
 *  remove virtual join support.
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

#ifndef _EXECNODES
#define _EXECNODES

#if defined ( WIN32 )
#include <windows.h>
#endif

#include <sqltypes.h>
#include "odbcnodes.h"

typedef int (*FETCH_FUNC)();
typedef int (*INNERFETCH_FUNC)( Handle_Stmt *, void * );

typedef struct Exec_TableRef
{
    int               	type;
    TableReference      *table_reference_node;
    DALTABLEINFO        dal_tableinfo;
    DALCOLUMNINFO       *dal_columninfo;
    int                 *column_needed;
    int                 *column_updatable;
    int                 *pk_columns;
    void                *sub_query_info;
    /*
     * check support for numerics in the dal layer
     */
    int                 numerics_checked;
    int                 has_numerics;

    int                 qual_checked;
    int                 qual_mask;
    int                 out_of_order_gd;
} Exec_TableRef;

typedef struct Exec_SelectRef
{
    int                 type;
    SelectSublist       *select_sl;
    int                 data_type;
    ColumnDataType      data_type_info;
    int                 data_type_set;
    Value               *value_from_sort;
    int                 set_to_null;
} Exec_SelectRef;

typedef struct GroupByInfo
{
    Expression          *expr;
    int                 size;
} GroupByInfo;

typedef struct TableIndexInfo
{
    int                 type;
    int                 done;
    int                 row_count;
    double              weight;
    LIST                index_list;
} TableIndexInfo;

typedef struct Exec_Select
{
    int                 type;
    int                 table_count;
    int                 select_list_count;
    int                 union_all;
    int                 distinct;
    SelectProlog        *query_spec;
    Exec_TableRef       **table_array;
    Exec_SelectRef      **select_array;
    Expression          *having;
    int                 updatable;
    LIST                set_value_list;
    LIST                having_set_value_list;
    struct Exec_Select  *next_union,*prev_union;
    struct Exec_Select  *parent;                    /* set to the enclosing select */
                                                    /* in a sub query */
    LIST                correlating_queries;        /* queries that this supplies */
                                                    /* correlated values to */
                                                    /* these need executing again */
                                                    /* and again */
    LIST                correlating_columns;        /* columns that are from another */
                                                    /* query that we can check to find */
                                                    /* if they have changed */
    LIST                sub_queries;                /* queries that this is the */
                                                    /* parent of */
    LIST                private_sub_queries;        /* Queries that appear in this */
                                                    /* stmts table list */
    int                 groupbycount;
    GroupByInfo         *groupby_info;
    Value               ***row_arrays;              /* array of pointers to array's of values pointers */
    Value               **row_long;                 /* array of record ids */
    char                **row_long_binary;
    LIST                parameter_list;    
    LIST                where_or_list;              /* list of parts of or's expr */
    void                *distinct_sort_handle;
    int                 distinct_sortlen;
    FETCH_FUNC          fetch_func;                 /* steps over unions */
    void                *exec_header;
    int                 keysettable;
    Handle_Stmt         *sstmt;
    int                 its_correlated;             /* don't execute at the beginning */
    LIST                order_by_list;            
    LIST                like_list;                  /* list of all LIKE stmts in the */
                                                    /* query */
    TableIndexInfo      *table_index_info;
    int                 unified_data_layer;         /* are all the tables in this query from the */
                                                    /* same DAL driver */
    int                 data_driver;                /* if so which one */
    DALITERATOR         query_iterator;             /* Iterator to execute relational query on */
    MEMHANDLE           relational_mh;              /* handle for relational work */
    int                 select_has_join;            /* at least one inner or outer join */
} Exec_Select;

typedef struct Exec_CatFunc
{
    int                 type;
    DALITERATOR         iterator;
    FETCH_FUNC          fetch_func;
    void                *sort_handle;
    char                *sort_buffer;
    int                 buffer_length;
} Exec_CatFunc;

typedef struct Exec_Header
{
    int                 type;
    int                 exec_count;
    LIST                exec_list;
    LISTITERATOR        list_iter;
} Exec_Header;

typedef struct TableQueryStruct
{
    Exec_TableRef        *table_reference;
    DALITERATOR          table_iterator;
    int                  fetched_but_not_read;
    int                  rewind_needed;             /* used for sub_q's in the select list */
                                                    /* signals that the next fetch needs to go to */
                                                    /* the first row */
    int                  next_first_in_join;        /* signal to fetch_joined_table to get the next */
                                                    /* row from the leading table of the join */
    int                  one_row_has_been_found;    /* at least one row from a join has been returned */
    int                  eod;
} TableQueryStruct;

typedef struct TableCry
{
    LIST                cry_list;
    int                 cry_count;
    QueryQualifier      **qualifiers;
} TableCry;

typedef struct ExecLet
{
    TableQueryStruct    *table_arr;
    TableQueryStruct    **effective_table_arr;
    Expression          *where;
    TableCry            *table_cry;
    TableCry            **effective_table_cry;
    int                 dead_end_query;
} ExecLet;

typedef struct ExecLet_Header
{
    int                 type;
    int                 count;
    int                 current_execlet;
    int                 table_count;
    MEMHANDLE           execl_memhandle;
    ExecLet             *execlet;
    INNERFETCH_FUNC     fetch_func;
    void                *groupby_sort_handle;
    int                 *groupby_key_pos;
    int                 groupby_sortlen;
    int                 groupby_grouplen;           /* the groupby columns at the */
                                                    /* start of the sort buffer */
    ColumnName          **groupby_col_arr;
    int                 *groupby_offset_arr;
    int                 *groupby_type_arr;
    int                 groupby_col_count;
    int                 groupby_table_count;
    int                 need_select_in_sort;
} ExecLet_Header;

typedef struct SetValue
{
    int                 type;
    int                 length;
    SetFunction         *function_node;
    void                *distinct_tree;
    Value               *value;
    int                 count;
} SetValue;

typedef struct Exec_CreateTable
{
    int                 type;
    DALTABLEINFO        table_info;
    LIST                column_def;
    LIST                table_constraint;
    LIST                primary_keys;
    char                *file_name;
    int                 key_number;
    
} Exec_CreateTable;

typedef struct Exec_CreateUser
{
    int                 type;
    char                *user_name;
    char                *password;
    char                *link;
} Exec_CreateUser;

typedef struct Exec_DropUser
{
    int                 type;
    char                *user_name;
    char                *link;
} Exec_DropUser;

typedef struct Exec_ColumnDefinition
{
    int                 type;
    char *              column_name;
    int                 base_type;
    int                 length;
    int                 fpart;
    int                 precision;
    int                 scale;
    Value               *default_value;
    int                 non_null;
    DALTABLEINFO        ref_dal_tableinfo;
    DALCOLUMNINFO       *ref_dal_columninfo;
    int                 ref_column_id;
    int                 with_ref;
    int                 ai_start;
    int                 ai_inc;
    char                *column_string;
    int                 key_number;
} Exec_ColumnDefinition;

typedef struct Exec_DropTable
{
    int                 type;
    DALTABLEINFO        table_info;
    int                 cascade_restrict;
} Exec_DropTable;

typedef struct Exec_DropIndex
{
    int                 type;
    DALTABLEINFO        index_info;
} Exec_DropIndex;

typedef struct Exec_CreatIndex
{
    int                 type;
    DALTABLEINFO        table_info;
    DALTABLEINFO        index_name;
    DALCOLUMNINFO       *column_info;
    LIST                column_def;
    int                 unique;
    int                 key_number;
} Exec_CreateIndex;

typedef struct Exec_ColumnIndex
{
    int                 type;
    DALCOLUMNINFO       *column_info;
    int                 direction;
} Exec_ColumnIndex;

typedef struct Exec_Insert
{
    int                 type;
    DALTABLEINFO        table_info;
    DALCOLUMNINFO       *column_info;
    LIST                value_list;
    LIST                parameter_list;
    Exec_Select         *query_stmt;
    Handle_Stmt         *sstmt;
    int                 unified_data_layer;         /* Can we use passthrough */
} Exec_Insert;

typedef struct Exec_ColumnDef
{
    int                 type;
    Expression          *ex;
    int                 def;
    DALCOLUMNINFO       *column_info;
    int                 column_id;
    Exec_Select         *query_ex;                  /* for set val=sub_query */
    Handle_Stmt         *sstmt;
} Exec_ColumnDef;

typedef struct Exec_DeleteSer
{
    int                 type;
    DALTABLEINFO        table_info;
    Exec_Select         *ex_select;
    Handle_Stmt         *stmt;
    LIST                parameter_list;
    int                 unified_data_layer;         /* Can we use passthrough */
} Exec_DeleteSer;

typedef struct Exec_DeletePos
{
    int                 type;
    Handle_Stmt         *target_stmt;
    int                 table_id;
} Exec_DeletePos;

typedef struct Exec_UpdateSer
{
    int                 type;
    DALTABLEINFO        table_info;
    DALCOLUMNINFO       *column_info;
    Exec_Select         *ex_select;
    LIST                value_list;
    LIST                parameter_list;
    Handle_Stmt         *stmt;
    int                 unified_data_layer;         /* Can we use passthrough */
} Exec_UpdateSer;

typedef struct Exec_UpdatePos
{
    int                 type;
    Handle_Stmt         *target_stmt;
    int                 table_id;
    LIST                value_list;
    LIST                parameter_list;
} Exec_UpdatePos;

typedef struct Using_Joined
{
    int                 left_ref;
    int                 right_ref;
} Using_Joined;

typedef struct Exec_Joined
{
    int                 type;
    QuerySpecification  *left_query;
    QuerySpecification  *right_query;
    QuerySpecification  *query;
    Exec_Select         *exec_select;
    int                 inner_outer;
    int                 left_right;
    Expression          *joining_expression;
    int                 exclude_count;
    int                 *blank_arr;
    int                 current_second_count;
    int                 in_full_join;                       /* we are doing the second half */
                                                            /* of a full join */
    void                *full_tree;                         /* id found so far for this join */
    int                 next_id;                            /* flag to get the next unused id */
    int                 found_this_id;                      /* how many matches have we had for */
                                                            /* this id */
} Exec_Joined;

typedef struct Exec_GrantRev
{
    int                 type;
    int                 priv;
    char                *user;    
    char                *column_name;
    int                 column_id;
    int                 with_grant_option;
    int                 cascade_or_restrict;
} Exec_GrantRev;

typedef struct Exec_Grant
{
    int                 type;
    DALTABLEINFO        table_info;
    DALCOLUMNINFO       *column_info;
    LIST                grant_list;
} Exec_Grant;

typedef struct Exec_Revoke
{
    int                 type;
    DALTABLEINFO        table_info;
    DALCOLUMNINFO       *column_info;
    LIST                revoke_list;
} Exec_Revoke;

typedef struct Exec_AlterDrop
{
    int                 type;
    DALTABLEINFO        table_info;
    DALCOLUMNINFO       *column_info;
    char                *column_name;
    int                 column_id;
    int                 cascade_or_restrict;
} Exec_AlterDrop;

typedef struct Exec_AlterAdd
{
    int                 type;
    DALTABLEINFO        table_info;
    DALCOLUMNINFO       *column_info;
    LIST                column_def;
    LIST                table_constraint;
    LIST                primary_keys;
} Exec_AlterAdd;

typedef struct Exec_CreateView
{
    int                 type;
    Exec_Select         *query_exec;
    QueryEx             *query;
    CreateView          *parse_stmt;
    DALTABLEINFO        table_info;
    char                **name_array;
    int                 name_array_count;
} Exec_CreateView;

typedef struct Exec_DropView
{
    int                 type;
    DALTABLEINFO        table_info;
} Exec_DropView;

typedef struct Correlating_Columns
{
    int                 type;
    ColumnName          *column;
    Value               *value;
} Correlating_Columns;

typedef struct Exec_Commit
{
    int                 type;
} Exec_Commit;

typedef struct Exec_Rollback
{
    int                 type;
} Exec_Rollback;

typedef struct Exec_Procedure
{
    int                 type;
    DALPROCINFO         proc_info;
    DALARGINFO          *arg_info;
    Expression          **arg_expression;
    int                 return_val;
    int                 result_cols;
    int                 in_out_args;
    LIST                parameter_list;
    DALITERATOR         iterator;
    FETCH_FUNC          fetch_func;
} Exec_Procedure;

#endif
