/*
 *  $Id: execute.h,v 1.18 2000/07/10 15:17:20 nick Exp $
 *
 *  $Log: execute.h,v $
 *  Revision 1.18  2000/07/10 15:17:20  nick
 *  Add 4 part names
 *
 *  Revision 1.17  2000/06/23 14:18:54  nick
 *  Add replication
 *
 *  Revision 1.16  2000/06/19 08:38:19  nick
 *   Add 1.4 support code
 *
 *  Revision 1.15  2000/03/08 14:25:51  nick
 *  Add missing '*'
 *
 *  Revision 1.14  2000/02/15 12:07:47  nick
 *  Add Procedure support
 *
 *  Revision 1.13  2000/01/12 14:39:33  nick
 *  Add tree based in evaluation
 *
 *  Revision 1.12  1999/09/22 11:33:14  nick
 *  Add change to check_ipd_type
 *
 *  Revision 1.11  1999/08/13 16:52:11  nick
 *  Add extra arg to expand_desc to cover bookmarks
 *
 *  Revision 1.10  1999/02/16 11:36:05  nick
 *  Add run_commit and run_rollback
 *
 *  Revision 1.9  1999/01/25 16:01:57  nick
 *  Add proto for refresh_correlated_querie
 *
 *  Revision 1.8  1998/12/03 17:22:35  nick
 *  add release_exec_private
 *
 *  Revision 1.7  1998/11/16 10:46:19  nick
 *  Add create_bookmark_fiels
 *
 *  Revision 1.6  1998/11/10 13:35:14  nick
 *  Add VIEW support
 *
 *  Revision 1.5  1998/10/20 11:07:51  nick
 *  Move set_pos to rowset.c
 *
 *  Revision 1.4  1998/09/23 17:46:16  nick
 *  change the prototype for put data
 *
 *  Revision 1.3  1998/09/21 14:22:06  nick
 *  Add long buffer functions.
 *
 *  Revision 1.2  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

#ifndef _EXECUTE
#define _EXECUTE

#if defined ( WIN32 )
#include <windows.h>
#endif

#include <setjmp.h>

#include "list.h"
#include "nodes.h"
#include "parsenodes.h"
#include "execnodes.h"

typedef struct TableDef
{
    char         *name;
    int            type;
    int            length;
    int            nullable;
} TableDef;

typedef struct TableQuery
{
    char        *catalog;
    int            catalog_length;
    char         *schema;
    int            schema_length;
    char         *name;
    int            name_length;
    char         *type;
    int            type_length;
} TableQuery;

typedef struct PKQuery
{
    char        *catalog;
    int            catalog_length;
    char         *schema;
    int            schema_length;
    char         *table_name;
    int            table_name_length;
} PKQuery, TablePrivilegesQuery;

typedef struct ColumnQuery
{
    char        *catalog;
    int            catalog_length;
    char         *schema;
    int            schema_length;
    char         *table_name;
    int            table_name_length;
    char         *column_name;
    int            column_name_length;
} ColumnQuery, PrivQuery;

typedef struct FKQuery
{
    char        *pk_catalog_name;
    int            pk_catalog_name_length;
    char         *pk_schema_name;
    int            pk_schema_name_length;
    char         *pk_table_name;
    int            pk_table_name_length;
    char        *fk_catalog_name;
    int            fk_catalog_name_length;
    char         *fk_schema_name;
    int            fk_schema_name_length;
    char         *fk_table_name;
    int            fk_table_name_length;
} FKQuery;

typedef struct SpecialColumnsQuery
{
    int            identifier_type;
    char        *catalog;
    int            catalog_length;
    char         *schema;
    int            schema_length;
    char         *table_name;
    int            table_name_length;
    int            scope;
    int            nullable;
} SpecialColumnsQuery;

typedef struct StatisticsQuery
{
    char        *catalog;
    int            catalog_length;
    char         *schema;
    int            schema_length;
    char         *table_name;
    int            table_name_length;
    int            unique;
    int            reserved;
} StatisticsQuery;

typedef struct ProcedureQuery
{
    char        *catalog;
    int            catalog_length;
    char         *schema;
    int            schema_length;
    char         *proc;
    int            proc_length;
} ProcedureQuery;

typedef struct ProcedureColQuery
{
    char        *catalog;
    int            catalog_length;
    char         *schema;
    int            schema_length;
    char         *proc;
    int            proc_length;
    char         *column;
    int            column_length;
} ProcedureColQuery;

typedef struct TypeInfoQuery
{
    int            data_type;
} TypeInfoQuery;

int load_next_exec( Handle_Stmt * );
int load_first_exec( Handle_Stmt * );
int load_first_sub_exec( Handle_Stmt *, Exec_Select * );
void release_exec( Handle_Stmt * );
void release_exec_private( Handle_Stmt * );
int generate_descriptors( Handle_Stmt * stmt, int api, TableDef * td, int col_count,
    int key_count, int * keys_order );
int query_catalog( Handle_Stmt *, int, void * );
int fetch_positioned( Handle_Stmt *, int orientation, int offset );
int initialise_exec( Handle_Stmt * );
int initialise_select_exec( Handle_Stmt * );
int run_create_table( Handle_Stmt * );
int run_drop_table( Handle_Stmt * );
int run_drop_index( Handle_Stmt * );
int run_create_index( Handle_Stmt * );
int run_insert( Handle_Stmt * );
int run_delete( Handle_Stmt * );
int run_update( Handle_Stmt * );
int run_update_pos( Handle_Stmt * );
int run_delete_pos( Handle_Stmt * );
int run_grant( Handle_Stmt * );
int run_revoke( Handle_Stmt * );
int run_alter_drop( Handle_Stmt * );
int run_alter_add( Handle_Stmt * );
int run_create_view( Handle_Stmt * );
int run_drop_view( Handle_Stmt * );
int run_commit( Handle_Stmt * );
int run_rollback( Handle_Stmt * );
int run_procedure( Handle_Stmt *stmt, int *rowset );
int run_createuser( Handle_Stmt * );
int run_dropuser( Handle_Stmt * );

void release_exec_select( Exec_Select *, Handle_Stmt * );
int create_pd( Handle_Desc * desc );
int release_pd( Handle_Desc * desc );
int expand_desc( Handle_Desc * desc, int count, int use_bookmarks );
void create_bookmark_field( Handle_Desc * desc );
int contract_desc( Handle_Desc * desc, int count, int use_bookmarks );
int check_ipd_type( int param_type, int value_type );
void setup_field( Desc_Field * field, int type );
void trim_desc( Handle_Desc * );
int init_set_value( void *, void * );
char * get_column_name_from_expr( void * expr );
char * get_table_from_expr( void * expr );
char * get_schema_from_expr( void * expr );
char * get_catalog_from_expr( void * expr );
char * get_link_from_expr( void * expr );
int end_current_transaction( void * dbc, int mode );
int copy_apd_to_ipd( Handle_Stmt * stmt );
int check_for_data_at_exec( Handle_Stmt *stmt, int **res );
int put_data( Handle_Stmt *stmt, void *data, int indicator );
int generate_target_value( Handle_Stmt *stmt, void *param_value_ptr );
int populate_ird( Handle_Stmt * stmt, Exec_Select * exec );
int get_display_size( int type, int length );
int get_octet_length( int type, int length );
int restart_single_exec( Handle_Stmt *, Exec_Select *, ExecLet_Header * );
int restart_execlet( ExecLet_Header * ex_head, int count, Handle_Stmt * stmt );
int refresh_correlated_querie( Exec_Select * ex, int *reset );
int init_replication( Handle_Stmt *stmt );

#endif
