/*
 *  $Id: memsys.h,v 1.7 2000/07/07 12:27:54 nick Exp $
 *
 *  $Log: memsys.h,v $
 *  Revision 1.7  2000/07/07 12:27:54  nick
 *  Rename mem_xyz fns to es_mem_xyz fns to avoid clashes with other software.
 *
 *  Revision 1.6  2000/03/08 16:32:41  nick
 *  Add C++ wrappers
 *
 *  Revision 1.5  1999/06/30 13:30:55  nick
 *  Extra tracing support
 *
 *  Revision 1.4  1998/12/03 17:23:06  nick
 *  add memory tracing
 *
 *  Revision 1.3  1998/11/03 16:05:41  nick
 *  Add mem_alloc_node
 *
 *  Revision 1.2  1998/09/21 14:23:01  nick
 *  Add support for callback at free function and usage counting.
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

/* memory allocation subsystem */

#ifndef _MEMSYS_H
#define _MEMSYS_H

#ifdef __cplusplus
extern "C" {             /* Assume C declarations for C++   */
#endif  /* __cplusplus */

#include "sysdefs.h"

typedef void * MEMHANDLE;

MEMHANDLE es_mem_alloc_handle( MEMHANDLE );
void * es_mem_alloc_ex( MEMHANDLE h, int size, void (*free_func)(void*));
void es_mem_release_handle( MEMHANDLE );

#ifdef _TRACE_MEMORY_USE
void * _mem_alloc( MEMHANDLE, int, int, char * );
#define es_mem_alloc(h,s) _mem_alloc(h,s,__LINE__,__FILE__)
#else
void * es_mem_alloc( MEMHANDLE, int );
#endif

#ifdef _TRACE_MEMORY_USE
void * _mem_alloc_node( MEMHANDLE, int, int, int, char * );
#define es_mem_alloc_node(h,s) _mem_alloc_node(h,s,__LINE__,__FILE__)
#else
void * es_mem_alloc_node( MEMHANDLE, int, int );
#endif

void es_mem_free( MEMHANDLE, void * );
void * es_mem_clone( MEMHANDLE h, void *data );

void es_mem_trace_init( void );
void es_mem_trace_term( void );

/*
 * rename
 */

#define mem_trace_init          es_mem_trace_init
#define mem_trace_term          es_mem_trace_term
#define mem_clone               es_mem_clone
#define mem_free                es_mem_free
#define mem_alloc_node          es_mem_alloc_node
#define mem_alloc               es_mem_alloc
#define mem_alloc_ex            es_mem_alloc_ex
#define mem_alloc_handle        es_mem_alloc_handle
#define mem_release_handle      es_mem_release_handle

#ifdef __cplusplus
}                                    /* End of extern "C" { */
#endif  /* __cplusplus */
#endif
