/*
 *  $Id: nodes.h,v 1.57 2001/06/22 08:56:32 nick Exp $
 *
 *  $Log: nodes.h,v $
 *  Revision 1.57  2001/06/22 08:56:32  nick
 *  Stop atoq being picked up on Linux boxes since GNU_SOURCE was defined.
 *
 *  Revision 1.56  2000/11/14 17:44:08  nick
 *  Add IS NULL support
 *
 *  Revision 1.55  2000/10/17 15:40:18  martin
 *  FreeBSD uses strtoq for string to bigint.
 *
 *  Revision 1.54  2000/08/14 13:16:58  nick
 *  Add join fixes
 *
 *  Revision 1.53  2000/07/13 15:17:59  nick
 *  Add option to use the local bigint funcs
 *
 *  Revision 1.52  2000/07/13 12:33:53  martin
 *  Add option to force definition of int64_t
 *
 *  Revision 1.51  2000/07/10 15:18:10  nick
 *  Add faster value definition
 *
 *  Revision 1.50  2000/06/19 14:17:46  nick
 *  Add CONSTRAINT_KEY
 *
 *  Revision 1.49  2000/06/19 14:07:07  nick
 *  Add CONSTRAINT_ON
 *
 *  Revision 1.48  2000/06/19 08:38:33  nick
 *   Add 1.4 support code
 *
 *  Revision 1.47  2000/06/14 12:01:50  martin
 *  Add constratint_nullable (nick)
 *
 *  Revision 1.46  2000/04/18 08:59:03  martin
 *  Do HAS_LINUXTYPES and HAS_INTTYPES.
 *
 *  Revision 1.45  2000/04/13 14:53:51  martin
 *  Add variations for true 64bit platforms.
 *
 *  Revision 1.44  2000/02/15 12:07:47  nick
 *  Add Procedure support
 *
 *  Revision 1.43  2000/02/04 11:29:11  martin
 *  For HAS_LINUXTYPES use sys/types.h not linux/types.h.
 *
 *  Revision 1.42  2000/01/27 14:21:10  nick
 *  Add linux/types
 *
 *  Revision 1.41  2000/01/27 14:18:42  tom
 *  fixed typo
 *
 *  Revision 1.40  2000/01/27 13:46:15  nick
 *  Add check to see if int64_t is native
 *
 *  Revision 1.39  2000/01/26 15:44:11  martin
 *  Add support for true 64bit platforms.
 *
 *  Revision 1.38  1999/12/22 14:29:57  nick
 *  Fix broken windows int64 typedef
 *
 *  Revision 1.37  1999/12/20 13:13:26  nick
 *  Add test for inttypes.h
 *
 *  Revision 1.36  1999/12/20 13:09:39  nick
 *  Use int64_t
 *
 *  Revision 1.35  1999/08/13 16:52:30  nick
 *  Add NewValue call
 *
 *  Revision 1.34  1999/06/30 13:31:12  nick
 *  Add support for auto increment
 *
 *  Revision 1.33  1999/05/25 10:18:51  nick
 *  Add NULL_FORCED_FLAG
 *
 *  Revision 1.32  1999/05/19 15:33:49  nick
 *  Fix WIN32 format string for bigints
 *
 *  Revision 1.31  1999/05/12 14:52:03  nick
 *  I don't believe It can take so many goes to produce a tolower replacement
 *
 *  Revision 1.30  1999/05/12 14:47:07  nick
 *  Fix TOLOWER
 *
 *  Revision 1.29  1999/05/12 13:56:51  nick
 *  Add TOLOWER
 *
 *  Revision 1.28  1999/05/12 13:20:11  nick
 *  Fix definition of ISSPACE
 *
 *  Revision 1.27  1999/05/12 12:30:50  nick
 *  Fix spelling on USE_BUILTIN_CTYPE
 *
 *  Revision 1.26  1999/05/12 11:03:59  nick
 *  Add CTYPE macros
 *
 *  Revision 1.25  1999/05/05 16:12:16  nick
 *  Move annoying bracket
 *
 *  Revision 1.24  1999/05/05 15:58:19  nick
 *  *** empty log message ***
 *
 *  Revision 1.23  1999/05/05 15:56:26  nick
 *  Alter STRTOLL def
 *
 *  Revision 1.22  1999/05/05 13:30:59  jason
 *  Add USE_STRTOLL option
 *
 *  Revision 1.21  1999/05/05 13:19:48  nick
 *  Add other extra arg to strtoll
 *
 *  Revision 1.20  1999/05/05 13:16:26  nick
 *  Add extra arg on strtoll
 *
 *  Revision 1.19  1999/05/04 13:14:45  nick
 *  Add BIGINTADDINT
 *
 *  Revision 1.18  1999/05/04 12:15:41  nick
 *  Add more bigint stuff
 *
 *  Revision 1.17  1999/04/27 08:28:17  nick
 *  Swap around the int in the 64 bit structure
 *
 *  Revision 1.16  1999/04/26 16:20:36  nick
 *  More native 64bit changes
 *
 *  Revision 1.15  1999/04/26 12:27:35  nick
 *  Add non native bigint support
 *
 *  Revision 1.14  1999/04/16 08:19:22  nick
 *  Flags added for the column usage
 *
 *  Revision 1.13  1999/03/04 22:25:33  nick
 *  Add extra sqi data types to Createtable
 *
 *  Revision 1.12  1999/02/16 11:38:19  nick
 *  Add support for Commit and Rollback
 *  Add support for CURRENT_TIME etc
 *
 *  Revision 1.11  1999/01/25 16:02:56  nick
 *  Add T_TableIndexInfo and T_Correlating_Columns structures
 *
 *  Revision 1.10  1998/12/15 10:31:00  nick
 *  Add T_CaseExpression, T_NullIfExpression, T_CoalesceExpression and T_WhenClause
 *
 *  Revision 1.9  1998/12/03 17:23:25  nick
 *  Add memory use tracing
 *
 *  Revision 1.8  1998/11/23 17:54:30  nick
 *  SQL_INVALID_TYPE can't be SQL_UNKNOWN_TYPE, they have slight different uses
 *
 *  Revision 1.7  1998/11/18 11:47:55  nick
 *  Alter SQL_INVALID_TYPE to be the same as SQL_UNKNOWN_TYPE
 *  the old value clashed with a interval type
 *
 *  Revision 1.6  1998/11/10 13:35:14  nick
 *  Add VIEW support
 *
 *  Revision 1.5  1998/10/20 09:28:59  nick
 *  Replaced BASE_FLOAT, BASE_INT, BASE_EXACT with BASE_NUMERIC
 *
 *  Revision 1.4  1998/09/21 14:23:47  nick
 *  Add interval support.
 *  Add long var support.
 *
 *  Revision 1.3  1998/08/18 15:29:43  nick
 *  Add default INTERVAL precisions
 *
 *  Revision 1.2  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

#ifndef NODES_H
#define NODES_H

#ifdef WIN32
typedef __int64        int64_t;
#elif defined( HAS_INTTYPES )
#include <inttypes.h>
#elif defined( HAS_LINUXTYPES )
#include <sys/types.h>
#elif defined ( NO_NATIVE64 )
typedef struct
{
    unsigned int    low_int;
    int             high_int;
} int64_t;
#else
typedef    long long     int64_t;
#endif


#if defined NEED_INT64_T
typedef struct
{
    unsigned int    low_int;
    int             high_int;
} int64_t;
#endif

/*
 * 64 support macros
 */

#if defined( NO_NATIVE64 ) || defined ( USE_LOCAL_BIGFUNC )

int64_t bigintaddint( int64_t, int );
int biginttoint( int64_t );
double biginttodbl( int64_t );
int bigintcmp( int64_t, int64_t );
int64_t dbltobigint( int64_t, double );
int64_t inttobigint( int64_t, int );
int64_t bigintmultint( int64_t, int );
int64_t bigintmultdbl( int64_t, double );
int64_t bigintdivdbl( int64_t, double );
int64_t bigintmodint( int64_t, int );
int64_t bigintdivint( int64_t, int );
int64_t string_to_bigint( char * );
int64_t bigintadd( int64_t, int64_t );
int64_t bigintsub( int64_t, int64_t );
int64_t bigintmul( int64_t, int64_t );
int64_t bigintdiv( int64_t, int64_t );
char *bigint_to_string( int64_t, char * );

# define BIGINTTOINT(a)      (biginttoint((a)))
# define BIGINTTODBL(a)      (biginttodbl((a)))
# define BIGINTCMP(a,b)      (bigintcmp((a),(b)))
# define DBLTOBIGINT(a,b)    (dbltobigint((a),(b)))
# define INTTOBIGINT(a,b)    (inttobigint((a),(b)))
# define BIGINTMULTINT(a,b)  (bigintmultint((a),(b)))
# define BIGINTADDINT(a,b)   (bigintaddint((a),(b)))
# define BIGINTMULTDBL(a,b)  (bigintmultdbl((a),(b)))
# define BIGINTDIVDBL(a,b)   (bigintdivdbl((a),(b)))
# define BIGINTMODINT(a,b)   (bigintmodint((a),(b)))
# define BIGINTDIVINT(a,b)   (bigintdivint((a),(b)))
# define STRING_TO_BIGINT(a) (string_to_bigint((a)))
# define BIGINTADD(a,b)      (bigintadd((a),(b)))
# define BIGINTSUB(a,b)      (bigintsub((a),(b)))
# define BIGINTMUL(a,b)      (bigintmul((a),(b)))
# define BIGINTDIV(a,b)      (bigintdiv((a),(b)))
# define BIGINT_TO_STRING(a,b) (bigint_to_string((a),(b)))
#else
# define BIGINTTOINT(a)      ((int)(a))
# define BIGINTTODBL(a)      ((double)(a))
# define BIGINTCMP(a,b)      ((a)>(b)?1:(a)<(b)?-1:0)
# define DBLTOBIGINT(a,b)    ((int64_t)(b))
# define INTTOBIGINT(a,b)    ((int64_t)(b))
# define BIGINTMULTINT(a,b)  ((a)*(b))
# define BIGINTADDINT(a,b)   ((a)+(b))
# define BIGINTMULTDBL(a,b)  ((a)*(int64_t)(b))
# define BIGINTDIVDBL(a,b)   ((a)/(int64_t)(b))
# define BIGINTMODINT(a,b)   ((a)%(b))
# define BIGINTDIVINT(a,b)   ((a)/(b))
# define BIGINTADD(a,b)      ((a)+(b))
# define BIGINTSUB(a,b)      ((a)-(b))
# define BIGINTMUL(a,b)      ((a)*(b))
# define BIGINTDIV(a,b)      ((a)/(b))
# if MJE__GNUC__ >= 2 && defined(MJE__USE_GNU) && !defined(MJE__STRICT_ANSI__)
#  define STRING_TO_BIGINT(a) (atoq((a)))
# elif defined ( WIN32 )
#  define STRING_TO_BIGINT(a) (_atoi64((a)))
# elif defined ( USE_STRTOLL )
#  define STRING_TO_BIGINT(a) (strtoll((a), NULL, 10))
# elif defined (__freebsd__)
#  define STRING_TO_BIGINT(a) (strtoq((a), NULL, 10))
# elif defined ( __IS64BIT__)
#  define STRING_TO_BIGINT(a) (atoi((a)))
# else
#  define STRING_TO_BIGINT(a) (atoll((a)))
# endif
# ifdef WIN32
#  define BIGINT_TO_STRING(a,b) (sprintf((b),"%I64d",(a)))
# else
#   ifdef __IS64BIT__
#     define BIGINT_TO_STRING(a,b) (sprintf((b),"%ld",(a)))
#   else
#     define BIGINT_TO_STRING(a,b) (sprintf((b),"%Ld",(a)))
#   endif
# endif
#endif

/*
 * the ctype funcs seem to be unreliable on some platforms
 */

#ifdef USE_BUILTIN_CTYPE
#define ISLOWER(c) ('a' <= (c) && (c) <= 'z')
#define TOUPPER(c) (ISLOWER(c) ? 'A' + ((c) - 'a') : (c))
#define TOLOWER(c) (ISLOWER(c) ? (c) : 'a' + ((c) - 'A'))
#define ISSPACE(c) ((c)==' '||(c)=='\t'||(c)=='\v'||(c)=='\f'||(c)=='\r')
#else
#define ISLOWER(c)  islower(c)
#define TOUPPER(c)  toupper(c)
#define TOLOWER(c)  tolower(c)
#define ISSPACE(c)  isspace(c)
#endif

/* defines for the parse nodes */

#define SQL_LITERAL_USER            2
#define SQL_LITERAL_CHARACTER        3
#define SQL_LITERAL_BIT                4
#define SQL_LITERAL_BINARY            5
#define SQL_LITERAL_DATE            6
#define SQL_LITERAL_TIME            7
#define SQL_LITERAL_TIMESTAMP        8
#define SQL_LITERAL_RAWCHARACTER    9
#define SQL_LITERAL_CURRENT_TIMESTAMP    10
#define SQL_LITERAL_CURRENT_TIME    11
#define SQL_LITERAL_CURRENT_DATE    12

#define SQL_SET_FUNC_COUNT            1
#define SQL_SET_FUNC_MAX            2
#define SQL_SET_FUNC_MIN            3
#define SQL_SET_FUNC_SUM            4
#define SQL_SET_FUNC_AVG            5
#define SQL_SET_COUNT_ASTER            6

#define SQL_DURATION_YEAR            1
#define SQL_DURATION_MONTH            2
#define SQL_DURATION_DAY            3
#define SQL_DURATION_HOUR            4
#define SQL_DURATION_MINUTE            5
#define SQL_DURATION_SECOND            6
#define SQL_DURATION_SECOND_PS        7    

/* these are negative to distinguish them from the ODBC functions */

#define SQL_SCALAR_CHAR_LENGTH        (-1)
#define SQL_SCALAR_OCTET_LENGTH        (-2)
#define SQL_SCALAR_POSITION            (-3)
#define SQL_SCALAR_SUBSTRING        (-4)
#define SQL_SCALAR_TRIM                (-5)
#define SQL_SCALAR_BIT_LENGTH        (-6)
#define SQL_INVALID_FUNCTION        (-99)

#define SQL_TRIM_TYPE_NONE            0x0000
#define SQL_TRIM_TYPE_LEADING        0x0001
#define SQL_TRIM_TYPE_TRAILING        0x0002
#define SQL_TRIM_TYPE_BOTH            ( SQL_TRIM_TYPE_LEADING | SQL_TRIM_TYPE_TRAILING )

/* arg type definitions */

#define SQL_FUNC_ARG_STRING            0x0001
#define SQL_FUNC_ARG_INTEGER        0x0002
#define SQL_FUNC_ARG_CHARACTER        0x0004
#define SQL_FUNC_ARG_NUMERIC        0x0008
#define SQL_FUNC_ARG_DATE            0x0010
#define SQL_FUNC_ARG_TIME            0x0020
#define SQL_FUNC_ARG_TIMESTAMP        0x0040
/*#define SQL_FUNC_ARG_TIMEINT        0x0080*/
#define SQL_FUNC_ARG_TEXT            0x0100
#define SQL_FUNC_ARG_VALUE            0x0200

#define SQL_FUNC_ARG_OPTIONAL        0x0800

/* extract types */

#define SQL_EXTRACT_YEAR            "0"
#define SQL_EXTRACT_YEAR_INT        0
#define SQL_EXTRACT_MONTH            "1"
#define SQL_EXTRACT_MONTH_INT        1
#define SQL_EXTRACT_DAY                "2"
#define SQL_EXTRACT_DAY_INT            2
#define SQL_EXTRACT_HOUR            "3"
#define SQL_EXTRACT_HOUR_INT        3
#define SQL_EXTRACT_MINUTE            "4"
#define SQL_EXTRACT_MINUTE_INT        4
#define SQL_EXTRACT_SECOND            "5"
#define SQL_EXTRACT_SECOND_INT        5

/* ODBC function id's */

#define SQL_FUNC_ABS                0
#define SQL_FUNC_ACOS                1
#define SQL_FUNC_ASCII                2
#define SQL_FUNC_ASIN                3
#define SQL_FUNC_ATAN                4
#define SQL_FUNC_ATAN2                5
#define SQL_FUNC_BIT_LENGTH            6
#define SQL_FUNC_CEILING            7
#define SQL_FUNC_CHAR                8
#define SQL_FUNC_CHARACTER_LENGTH    9
#define SQL_FUNC_CONCAT                10
#define SQL_FUNC_CONVERT            11
#define SQL_FUNC_COS                12
#define SQL_FUNC_COT                13
#define SQL_FUNC_CURDATE            14
#define SQL_FUNC_CURTIME            15
#define SQL_FUNC_CURTIMESTAMP        16
#define SQL_FUNC_DATABASE            17
#define SQL_FUNC_DAYNAME            18
#define SQL_FUNC_DAYOFMONTH            19
#define SQL_FUNC_DAYOFWEEK            20
#define SQL_FUNC_DAYOFYEAR            21
#define SQL_FUNC_DEGREES            22
#define SQL_FUNC_DIFFERENCE            23
#define SQL_FUNC_EXP                24
#define SQL_FUNC_EXTRACT            25
#define SQL_FUNC_FLOOR                26
#define SQL_FUNC_HOUR                27
#define SQL_FUNC_IFNULL                28
#define SQL_FUNC_INSERT                29
#define SQL_FUNC_LCASE                30
#define SQL_FUNC_LEFT                31
#define SQL_FUNC_LENGTH                32
#define SQL_FUNC_LOCATE                33
#define SQL_FUNC_LOG                34
#define SQL_FUNC_LOG10                35
#define SQL_FUNC_LTRIM                36
#define SQL_FUNC_MINUTE                37
#define SQL_FUNC_MOD                38
#define SQL_FUNC_MONTH                39
#define SQL_FUNC_MONTHNAME            40
#define SQL_FUNC_NOW                41
#define SQL_FUNC_OCTET_LENGTH        42
#define SQL_FUNC_PI                    43
#define SQL_FUNC_POSITION            44
#define SQL_FUNC_POWER                45
#define SQL_FUNC_QUARTER            46
#define SQL_FUNC_RADIANS            47
#define SQL_FUNC_RAND                48
#define SQL_FUNC_REPEAT                49
#define SQL_FUNC_REPLACE            50
#define SQL_FUNC_RIGHT                51
#define SQL_FUNC_ROUND                52
#define SQL_FUNC_RTRIM                53
#define SQL_FUNC_SECOND                54
#define SQL_FUNC_SIGN                55
#define SQL_FUNC_SIN                56
#define SQL_FUNC_SOUNDEX            57
#define SQL_FUNC_SPACE                58
#define SQL_FUNC_SQRT                59
#define SQL_FUNC_SUBSTRING            60
#define SQL_FUNC_TAN                61
#define SQL_FUNC_TIMESTAMPADD        62
#define SQL_FUNC_TIMESTAMPDIFF        63
#define SQL_FUNC_TRUNCATE            64
#define SQL_FUNC_UCASE                65
#define SQL_FUNC_USER                66
#define SQL_FUNC_WEEK                67
#define SQL_FUNC_YEAR                68

/* join types */

#define SQL_JOIN_DEFAULT            0
#define SQL_JOIN_NATURAL            1
#define SQL_JOIN_INNER                2
#define SQL_JOIN_OUTER                3
#define SQL_JOIN_LEFT                4
#define SQL_JOIN_LEFT_OUTER            5
#define SQL_JOIN_RIGHT                6
#define SQL_JOIN_RIGHT_OUTER        7
#define SQL_JOIN_FULL                8

/* expr node types */

#define BOOL_NOT                    1
#define BOOL_AND                    2
#define BOOL_OR                        3
#define EXPR_OP                        4

#define SQL_NOTYPE                  (-10000)

#define OPERATOR_NULL                   0
#define OPERATOR_PLUS                   1
#define OPERATOR_MINUS                  2
#define OPERATOR_CONCAT                 3
#define OPERATOR_MULT                   4
#define OPERATOR_DIV                    5
#define OPERATOR_UMINUS                 6
#define OPERATOR_LT                     7
#define OPERATOR_GT                     8
#define OPERATOR_GTE                    9
#define OPERATOR_LTE                    10
#define OPERATOR_EQ                     11
#define OPERATOR_NE                     12

/* for the query optimiser */

#define OPERATOR_BETWEEN                13
#define OPERATOR_LIKE                   14
#define OPERATOR_ISNULL                 15

/* on delete option */

#define SQL_ON_DELETE_NO_ACTION        0
#define SQL_ON_DELETE_CASCADE        1
#define SQL_ON_DELETE_SET_NULL        2
#define SQL_ON_DELETE_SET_DEFAULT    3

/* table and column constraint types */

#define SQL_CONSTRAINT_UNIQUE           1
#define SQL_CONSTRAINT_PRIMARY          2
#define SQL_CONSTRAINT_CHECK            3
#define SQL_CONSTRAINT_FOREIGN          4
#define SQL_CONSTRAINT_NOT_NULL         5
#define SQL_CONSTRAINT_REFERENCES       6
#define SQL_CONSTRAINT_DEFAULT          7
#define SQL_CONSTRAINT_AUTOINC          8
#define SQL_CONSTRAINT_NULLABLE         9 
#define SQL_CONSTRAINT_AS               10
#define SQL_CONSTRAINT_ON               11
#define SQL_CONSTRAINT_KEY              12

/* column specification types */

#define SQL_CS_NULL                    1
#define SQL_CS_DEFAULT                2

/* priviliges */

#define SQL_PRIVILEGE_ALL            0
#define SQL_PRIVILEGE_DELETE        1
#define SQL_PRIVILEGE_INSERT        2
#define SQL_PRIVILEGE_SELECT        3
#define SQL_PRIVILEGE_UPDATE        4
#define SQL_PRIVILEGE_REFERENCES    5

#define    SQL_DEFAULT_ACTION            0

#define SQL_ASCENDING                1
#define SQL_DESCENDING                2

#define SQL_ACTION_CASCADE            1
#define SQL_ACTION_RESTRICT            2

#define SQL_ALL                        1
#define SQL_DISTINCT                2

#define SQL_QUANTIFIED_ALL            1
#define SQL_QUANTIFIED_ANY            2
#define SQL_QUANTIFIED_SOME            3

/* Value types */

#define VALUE_TYPE_INTEGER              1
#define VALUE_TYPE_DOUBLE               2
#define VALUE_TYPE_STRING               3
#define VALUE_TYPE_BIT                  4
#define VALUE_TYPE_BINARY               5
#define VALUE_TYPE_PARAMETER            6
#define VALUE_TYPE_DATE                 7
#define VALUE_TYPE_TIME                 8
#define VALUE_TYPE_TIMESTAMP            9
#define VALUE_TYPE_NUMERIC              10
#define VALUE_TYPE_NULL                 11
#define VALUE_TYPE_BIGINT               12
#define VALUE_TYPE_INTERVAL_YEAR        13         
#define VALUE_TYPE_INTERVAL_DAY         14         
#define VALUE_TYPE_CURRENT_DATE         15
#define VALUE_TYPE_USER                 16
#define VALUE_TYPE_CURRENT_TIME         17
#define VALUE_TYPE_CURRENT_TIMESTAMP    18

#define VALUE_TYPE_PINT4                19
#define VALUE_TYPE_PINT2                20
#define VALUE_TYPE_PINT1                21
#define VALUE_TYPE_PDOUBLE              22
#define VALUE_TYPE_PDATE                23
#define VALUE_TYPE_PTIME                24
#define VALUE_TYPE_PTIMESTAMP           25
#define VALUE_TYPE_PINTERVAL            26
#define VALUE_TYPE_PNUMERIC             27
#define VALUE_TYPE_PBIGINT              28

#define VALUE_TYPE_LONGSTRING           29
#define VALUE_TYPE_LONGBINARY           30

/*
 * special Value with built in char field
 */

#define BIG_VALUE_EXTENSION             128

/*
 * is a column in use in a row set
 */

#define COLUMN_NOT_NEEDED               0
#define COLUMN_NEEDED                   1
#define COLUMN_NEEDED_IN_KEYSET         2

/* extra pseudo SQL types */

#define SQL_NULL_TYPE                   99
#define SQL_BOOL_TYPE                   100
#define SQL_INVALID_TYPE                -9999

#define SQL_SEQUENCE_ERROR              -9999

#define BASE_STRING_TYPE                0
#define BASE_NUMBER_TYPE                1
#define BASE_BINARY_TYPE                2
#define BASE_DATE_TYPE                  3
#define BASE_TIME_TYPE                  4
#define BASE_TIMESTAMP_TYPE             5
#define BASE_INTERVAL_YEAR_TYPE         6
#define BASE_INTERVAL_DAY_TYPE          7

#define TRUE                            1
#define FALSE                           0

#define DEFAULT_LEADING_PRECISION       2
#define DEFAULT_SECONDS_PRECISION       6


/*
 * flag added to isnull bin in value to indicate that the value has
 * been set to null by a failed outer join, but this enables the original
 * null, non null value to be retained
 */

#define NULL_FORCED_FLAG                 0x0200

typedef enum NodeTag
{
    T_Invalid = 0,

    /* Parse tree nodes */

    T_AlterTableAdd = 100,
    T_AlterTableDrop,
    T_CreateIndex,
    T_CreateTable,
    T_CreateView,
    T_DeletePositioned,
    T_DeleteSearched,
    T_DropIndex,
    T_DropTable,
    T_DropView,    
    T_Grant,                    /* 110 */
    T_Insert,
    T_Revoke,
    T_SelectStmt,
    T_SelectForUpdate,
    T_SelectProlog,
    T_UpdatePositioned,
    T_UpdateSearched,
    T_Procedure,
    T_TableName,
    T_ColumnDefinition,         /* 120 */
    T_Identifier,
    T_QuerySpecification,
    T_QueryEx,
    T_InsertSource,
    T_TableReference,
    T_SetColumnIdentifier,
    T_IndexColDef,
    T_ColumnElement,
    T_Privilege,
    T_SelectSublist,            /* 130 */
    T_JoinedTable,              /* Not Used */
    T_ColumnName,
    T_SortSpecification,
    T_DataType,
    T_ColConstDef,
    T_TableConstraint,
    T_ColumnSpec,
    T_QualifiedJoin,
    T_PrecisionScale,
    T_IntervalQualifier,        /* 140 */
    T_IntervalField,
    T_CastExpression,
    T_ScalarFunction,
    T_SetFunction,
    T_TrimExpression,
    T_ExistsPredicate,
    T_LikePredicate,
    T_NullPredicate,
    T_OverlapsPredicate,        /* Not used */
    T_InPredicate,              /* 150 */
    T_QuantifiedPredicate,
    T_Expression,
    T_List,
    T_Value,
    T_Parameter,
    T_IntervalValue,
    T_CaseExpression,
    T_NullIfExpression,
    T_CoalesceExpression,
    T_WhenClause,               /* 160 */
    T_Commit,
    T_Rollback,
    T_DataDefinition,
    T_CreateUser,
    T_DropUser,
    T_BigValue,
    T_JoinType,
    T_JoinSpecification,

    /* ODBC Handle Nodes */
    
    T_Handle_Env = 200,
    T_Handle_Dbc,
    T_Handle_Stmt,
    T_Handle_Desc,
    T_Desc_Field,

    /* Diagnostic Nodes */

    T_Error_Header = 300,
    T_Error_Record,

    /* Execution Nodes */

    T_Exec_Select = 400,
    T_Exec_TableRef,
    T_Exec_SelectRef,
    T_Exec_Header,
    T_Exec_CatFunc,
    T_ExecLet_Header,
    T_SetValue,
    T_Exec_CreateTable,
    T_Exec_ColumnDefinition,
    T_Exec_DropTable,
    T_Exec_DropIndex,   /* 410 */
    T_Exec_CreateIndex,
    T_Exec_ColumnIndex,
    T_Exec_Insert,
    T_Exec_ColumnDef,
    T_Exec_DeleteSer,
    T_Exec_UpdateSer,
    T_Exec_DeletePos,
    T_Exec_UpdatePos,
    T_Exec_Joined,
    T_Using_Joined,     /* 420 */
    T_Exec_Grant,
    T_Exec_Revoke,
    T_Exec_GrantRev,
    T_Exec_AlterDrop,
    T_Exec_AlterAdd,
    T_Exec_CreateView,
    T_Exec_DropView,
    T_TableIndexInfo,
    T_Correlating_Columns,
    T_Exec_Commit,      /* 430 */
    T_Exec_Rollback,
    T_Exec_Procedure,
    T_Exec_CreateUser,
    T_Exec_DropUser
} NodeTag;

/*
 * special allocation function for a Value
 * to allow the sval to point to the same bit of memory as the Node
 */

void * NewValue( void * mh, int ext );

typedef struct Node {
    int            type;
    double        this_causes_alligment_on_alphas;
} Node;

#define nodeTag(_node_)            ((Node*)_node_)->type

#define MakeNode(_node_,mh)        (_node_*)newNode(sizeof(_node_),T_##_node_,mh);

#ifdef _TRACE_MEMORY_USE

#define newNode( s,t,h )            _newNode(s,t,h,__LINE__,__FILE__ )
#define NewValue( a, b )            _NewValue( a, b, __LINE__, __FILE__ )

extern Node *_newNode( int size, NodeTag tag, void * mh, int line, char *file );

#else

extern Node *newNode( int size, NodeTag tag, void * mh );

#endif


/* 
 * extract the exec node from a sub stmt, based on it being a sub select or a 
 * join
 */

void * extract_select( void * );

#endif
