/*
 *  $Id: odbcerror.h,v 1.16 2000/02/15 12:07:47 nick Exp $
 *
 *  $Log: odbcerror.h,v $
 *  Revision 1.16  2000/02/15 12:07:47  nick
 *  Add Procedure support
 *
 *  Revision 1.15  1999/05/10 08:02:22  nick
 *  Add colon to general error message to improve readability
 *
 *  Revision 1.14  1999/04/16 08:20:07  nick
 *  Add S1107, S1108, S1C00
 *
 *  Revision 1.13  1999/03/08 13:30:05  nick
 *  Add SQL_ERROR_22012 division by zero
 *
 *  Revision 1.12  1999/02/16 11:38:57  nick
 *  Add SetErrorCount function
 *
 *  Revision 1.11  1999/01/25 16:03:36  nick
 *  Spelling mistake
 *  ,
 *
 *  Revision 1.10  1999/01/04 13:03:02  nick
 *  Add odbc_version to structure to enable SQLState mapping
 *
 *  Revision 1.9  1998/12/03 17:23:48  nick
 *  Add IM001
 *
 *  Revision 1.8  1998/11/18 11:49:01  nick
 *  Add HYC00
 *
 *  Revision 1.7  1998/11/03 16:06:08  nick
 *  Add 01S02, 42000 and 21S02
 *
 *  Revision 1.6  1998/10/20 09:29:46  nick
 *  Added a couple more error's relating to concurrency and bookmarks
 *
 *  Revision 1.5  1998/09/21 14:24:30  nick
 *  Change some ISO 9075 mssages to be ODBC 3.0 messages.
 *  Add errors for interval support.
 *
 *  Revision 1.4  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.3  1998/05/26 22:01:43  sources
 *  put it back to add endtrans support
 *
 *  Revision 1.1  1998/05/26 21:53:02  sources
 *  Initial revision
 *
 *  Revision 1.2  1998/05/08 15:32:54  nick
 *  Change prototype from long * to SQLINTEGER *
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

#ifndef _ODBCERROR
#define _ODBCERROR

#if defined ( WIN32 )
#include <windows.h>
#endif

typedef void * ERRORHANDLE;

ERRORHANDLE CreateErrorHeader( MEMHANDLE, void * );
void FreeErrorHeader( ERRORHANDLE );
void SetupErrorHeader( ERRORHANDLE h, int func_code );
SQLRETURN SetReturnCode( ERRORHANDLE h, int ret_code );
void PostError( ERRORHANDLE h, int class_origin, int col_number,
    char * driver_name, int native, int row_number, char * subclass,
    char * sqlstate, char * text, ... );
void FreeErrorHeader( ERRORHANDLE h );
int GetDiagRec( ERRORHANDLE h, int rec_number, char * sqlstate, 
    SQLINTEGER * native_error, char * message, int buffer_length, short * text_length_ptr );
int GetDiagField( int type, ERRORHANDLE handle, int record_number,
    int diag_ident, void * diag_ptr, int buff_length, short * len_ptr );    
void FinalError( void * handle, int error, int ret_code );
void AddError( void * handle, int error, int ret_code );
void PostDalError( SQLHANDLE handle, char * driver_name, 
    int native, char * sqlstate, char * text );
void SetErrorCount( ERRORHANDLE handle, int count );

#define SQL_ERROR_CLASS_ODBC                1
#define SQL_ERROR_CLASS_XOPEN                2

/* error definitions */

#define SQL_ERROR_01004                "ISO 9075", "01004", "String data, right truncated"
#define SQL_ERROR_01S02                "ODBC 3.0", "01S02", "Option value changed"
#define SQL_ERROR_01S07                "ODBC 3.0", "01S07", "Fractional truncation"
#define SQL_ERROR_01S09                "ISO 9075", "01S09", "Invalid keyword"

#define SQL_ERROR_40001                "ISO 9075", "40001", "Serialization failure"

#define SQL_ERROR_42001                "ISO 9075", "42000", "Syntax error or access violation"

#define SQL_ERROR_07005                "ISO 9075", "07005", "Prepared statement not a cursor-specification"
#define SQL_ERROR_07006                "ISO 9075", "07006", "restricted data type attribute violation"
#define SQL_ERROR_07009                "ISO 9075", "07009", "Invalid descriptor index"

#define SQL_ERROR_08002                "ISO 9075", "08002", "Connection name in use"
#define SQL_ERROR_08003                "ISO 9075", "08003", "Connection does not exist"

#define SQL_ERROR_21S02                "ISO 9075", "21S02", "Degree of derived table does not match column list"

#define SQL_ERROR_22001                "ISO 9075", "22001", "String data, right truncated"
#define SQL_ERROR_22002                "ISO 9075", "22002", "Indicator variable required but not supplied"
#define SQL_ERROR_22003                "ISO 9075", "22003", "Numeric value out of range"
#define SQL_ERROR_22007                "ISO 9075", "22007", "Invalid datetime format"
#define SQL_ERROR_22008                "ISO 9075", "22008", "Data value is not a valid datetime"
#define SQL_ERROR_22012                "ISO 9075", "22012", "Division by zero"
#define SQL_ERROR_22015                "ISO 9075", "22015", "Interval field overflow"

#define SQL_ERROR_24000                "ISO 9075", "24000", "Invalid cursor state"

#define SQL_ERROR_25000                "ISO 9075", "25000", "Invalid transaction state"
#define SQL_ERROR_25S01                "ODBC 3.0", "25S01", "Transaction state unknown"

#define SQL_ERROR_34000                "ISO 9075", "34000", "Invalid cursor name"
#define SQL_ERROR_3C000                "ISO 9075", "3C000", "Duplicate cursor name"

#define SQL_ERROR_42000                "ISO 9075", "42000", "Syntax error or access violation \"%s\""
#define SQL_ERROR_42S02                "ISO 9075", "42S02", "Base table or view %s not found"
#define SQL_ERROR_42S02a               "ISO 9075", "42S02", "Stored Procedure %s not found"
#define SQL_ERROR_42S22                "ISO 9075", "42S22", "Column %s not found in table %s"

#define SQL_ERROR_HY000                "ISO 9075", "HY000", "General error: %s"
#define SQL_ERROR_HY001                "ISO 9075", "HY001", "Memory allocation error"
#define SQL_ERROR_HY007                "ISO 9075", "HY007", "Associated statement is not prepared"
#define SQL_ERROR_HY008                "ISO 9075", "HY008", "Operation Canceled"
#define SQL_ERROR_HY009                "ISO 9075", "HY009", "Invalid use of null pointer"
#define SQL_ERROR_HY010                "ISO 9075", "HY010", "Function sequence error"
#define SQL_ERROR_HY011                "ISO 9075", "HY011", "Attribute cannot be set now"
#define SQL_ERROR_HY012                "ISO 9075", "HY012", "Invalid attribute/option identifier"
#define SQL_ERROR_HY017                "ISO 9075", "HY017", "Invalid use of a automatically allocated descriptor handle"
#define SQL_ERROR_HY019                "ISO 9075", "HY019", "Non-character and non-binary data sent in pieces"
#define SQL_ERROR_HY020                "ISO 9075", "HY020", "Attempt to concatenate a null value"
#define SQL_ERROR_HY024                "ISO 9075", "HY024", "Invalid attribute value"
#define SQL_ERROR_HY090                "ISO 9075", "HY090", "Invalid string or buffer length"
#define SQL_ERROR_HY091                "ISO 9075", "HY091", "Invalid descriptor field identifier"
#define SQL_ERROR_HY092                "ISO 9075", "HY092", "Option type out of range"
#define SQL_ERROR_HY096                "ISO 9075", "HY096", "Information type out of range"
#define SQL_ERROR_HY106                "ISO 9075", "HY106", "Fetch type out of range"
#define SQL_ERROR_HY109                "ODBC 3.0", "HY109", "Invalid cursor position"
#define SQL_ERROR_HY111                "ODBC 3.0", "HY111", "Invalid bookmark value"

#define SQL_ERROR_HYC00                "ODBC 3.0", "HYC00", "Optional feature not implemnted"

#define SQL_ERROR_HYT00                "ODBC 3.0", "HYT00", "Timeout Expired"

#define SQL_ERROR_IM001                "ODBC 3.0", "IM001", "Driver does not support this function"
#define SQL_ERROR_IM002                "ODBC 3.0", "IM002", "Data source name not found and no default driver specified"

#define SQL_ERROR_S1107                "ODBC 2.0", "S1107", "Row value out of range"
#define SQL_ERROR_S1108                "ODBC 2.0", "S1108", "Concurrency option out of range"
#define SQL_ERROR_S1C00                "ODBC 2.0", "S1C00", "Driver no capable"

#endif
