/*
 *  $Id: odbcnodes.h,v 1.33 2000/09/26 17:11:26 nick Exp $
 *
 *  $Log: odbcnodes.h,v $
 *  Revision 1.33  2000/09/26 17:11:26  nick
 *  Add flag to indicate that the ipd needs repopulating
 *
 *  Revision 1.32  2000/08/22 10:01:18  nick
 *  Add callbacks for long functions
 *
 *  Revision 1.31  2000/08/18 15:35:31  nick
 *  move extract_deferred again
 *
 *  Revision 1.30  2000/08/18 11:53:34  nick
 *  Add deferred read support
 *
 *  Revision 1.29  2000/08/01 08:39:38  nick
 *  Remove highest_needed
 *
 *  Revision 1.28  2000/07/31 11:31:19  nick
 *  Add flag to show highest used descriptor field
 *
 *  Revision 1.27  2000/07/28 15:17:55  nick
 *  Add extra value to the field structute to mark if it is needed
 *
 *  Revision 1.26  2000/07/18 09:44:54  nick
 *  Add extra args to sql92_parse
 *
 *  Revision 1.25  2000/06/23 14:19:08  nick
 *  Add replication
 *
 *  Revision 1.24  2000/06/06 12:04:43  nick
 *  remove numerics_checked
 *
 *  Revision 1.23  2000/06/05 08:19:19  nick
 *  Add pasthrough support
 *
 *  Revision 1.22  2000/02/17 11:49:53  nick
 *  Add values to statement handle to indicate if a sqi driver supports numerics
 *
 *  Revision 1.21  2000/01/12 14:39:14  nick
 *  Add tree based in evaluation
 *
 *  Revision 1.20  2000/01/12 13:51:52  nick
 *  Add new info extraction elements
 *
 *  Revision 1.19  1999/09/22 11:34:11  nick
 *  Add support for win32 threads
 *
 *  Revision 1.18  1999/05/25 10:19:14  nick
 *  Make literal_prefix literal_suffix local_type_name and type_name arrays
 *
 *  Revision 1.17  1999/04/16 08:20:30  nick
 *  Add rowset_size
 *
 *  Revision 1.16  1999/03/17 17:18:37  nick
 *  Add schema_root to the DBC
 *
 *  Revision 1.15  1999/02/16 11:39:21  nick
 *  Add reghandle for registry operations
 *  Save the current param in a ParamData PutData operation
 *
 *  Revision 1.14  1999/01/25 16:04:00  nick
 *  Add refresh_correlated_querie callback
 *  Add flag to allow set function updating
 *
 *  Revision 1.13  1999/01/04 13:06:49  nick
 *  Add extra callbacks
 *  Add flags for cancel and timeouts
 *
 *  Revision 1.12  1998/12/15 10:31:32  nick
 *  Add extra arg to compile_like
 *
 *  Revision 1.11  1998/12/04 11:27:03  nick
 *  Add eval_cast callback
 *
 *  Revision 1.10  1998/12/03 17:24:09  nick
 *  Add the check_expression callback
 *  Add async support
 *
 *  Revision 1.9  1998/11/20 13:19:41  nick
 *  Add the param_offset field for arrays of parameters
 *
 *  Revision 1.8  1998/11/18 11:49:22  nick
 *  remove unwanted fields from the statement, they should have been in the
 *  descriptor
 *
 *  Revision 1.7  1998/11/16 10:47:09  nick
 *  Fix bug where select count(*) from a empty table returned
 *  no rows.
 *
 *  Revision 1.6  1998/11/10 13:35:14  nick
 *  Add VIEW support
 *
 *  Revision 1.5  1998/11/03 16:06:59  nick
 *  Add bookmark and setpos support
 *
 *  Revision 1.4  1998/10/20 09:30:24  nick
 *  Added bookmark and in_extended_fetch members of the stmt struct
 *
 *  Revision 1.3  1998/09/21 14:25:48  nick
 *  Add fields to support data at exec.
 *
 *  Revision 1.2  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

#ifndef _ODBCNODES
#define _ODBCNODES

#if defined ( WIN32 )
#include <windows.h>
#endif

#include "time.h"

#include "dal.h"
#include "odbcerror.h"
#include "parsenodes.h"
#include "cursor.h"
#include "connfunc.h"
#include "threads.h"

typedef struct Handle_Env
{
    int                     type;
    MEMHANDLE               root_mem_handle;
    int                     odbc_version;
    int                     output_nts;
    ERRORHANDLE             error_header;
    int                     state;
    int                     connection_count;
    void                    *reghandle;
} Handle_Env;

typedef struct Handle_Dbc
{
    int                     type;
    MEMHANDLE               root_mem_handle;
    Handle_Env              *env;
    ERRORHANDLE             error_header;
    int                     state;
    int                     statement_count;
    void                    *stmt_list;
    int                     async_enable;
    int                     enable_auto_ipd;
    int                     access_mode;
    int                     autocommit;
    int                     connection_timeout;
    char                    *current_catalog;
    int                     login_timeout;
    int                     metadata_id;
    int                     odbc_cursors;
    int                     packet_size;
    void                    *quiet_mode;
    int                     trace;
    char                    *tracefile;
    char                    *translate_lib;
    int                     translate_option;
    int                     txn_isolation;
    DALHANDLE               dalhandle;
    LIST                    cursor_list;
    int                     cursor_count;
    struct con_struct       con_str;
    void                    *schema_root;

    /* 
     * the following are function pointers that allow the DAL driver 
     * to call back up the stack to the upper layers to perform VIEW's
     */

    int                     (*create_pd)(void*);
    void                    (*setup_descriptor_defaults)(void*,int);
    SQLRETURN               (*LocalSQLAllocDesc)(void*,void*,void*,int);
    int                     (*init_stmt)(void*,void*);
    int                     (*load_first_exec)(void*);
    int                     (*sql92_validate)(void*);
    int                     (*sql92_free_handle)(void*);
    char *                  (*sql92_geterror)(void*);
    int                     (*sql92_parse)(void*,char*,int,void**);
    void *                  (*sql92_alloc_handle)(void*);
    int                     (*term_stmt)(void*);
    int                     (*LocalFreeDesc)(void*);
    int                     (*release_pd)(void*);
    int                     (*release_exec)(void*);
    int                     (*fetch_positioned)(void*,int,int);
    int                     (*extract_data)(void*,int,int,int,void*,int,void*,void*,int);
    int                     (*initialise_exec)(void*);
    int                     (*check_like)(void*,void*);
    int                     (*compile_like)(void*,void*,void*);
    int                     (*check_expression)(int*,void*,void*);
    void *                  (*eval_cast)(void*,void*,void*);
    int                     (*setup_tables)(void*);
    int                     (*setup_cpriv)(void*);
    int                     (*setup_columns)(void*);
    int                     (*setup_indexes)(void*,char*,char*,char*);
    int                     (*setup_tpriv)(void*);
    int                     (*setup_views)(void*);
    int                     (*get_info)(void*,int,void*,int,short*);
    int                     (*refresh_correlated_querie)(void*,int*);
    void *                  (*extract_deferred)(void*);
    int                     (*extract_from_long_buffer)(void*,char*,long,int*);
    void                    (*rewind_long_buffer)(void*);

    /*
     * magic valus MS added for Access 2000
     */
    char                    *ms_30002;

    /*
     * potential interface to ini files
     */
    void                    *ini_handle;
    void                    *get_ini_str;

    /*
     * default values for statement attr
     */
    SQLHANDLE               default_stmt;

    /*
     * replication support
     */
     LIST                    replication_list;
     char                    rep_file[ 128 ];

} Handle_Dbc;

typedef struct Handle_Stmt
{
    int                     type;
    MEMHANDLE               root_mem_handle;
    Handle_Env              *env;
    Handle_Dbc              *dbc;
    ERRORHANDLE             error_header;
    void                    *executor;            /* current executing info */
    int                     state;
    int                     prepared;
    SQLHANDLE               app_row, app_param, imp_row, imp_param;
    SQLHANDLE               cur_app_row, cur_app_param, cur_imp_row, cur_imp_param;
    int                     concurrency;
    int                     scrollable;
    int                     sensitivity;
    int                     cursor_type;
    int                     enable_auto_ipd;
    SQLPOINTER              current_bookmark;
    int                     keyset_size;
    int                     max_length;
    int                     max_rows;
    int                     metadata_id;
    int                     noscan;
    int                     query_timeout;
    int                     retrieve_data;
    int                     simulate_cursor;
    int                     row_number;
    int                     use_bookmarks;
    void                    *parsehandle;
    MEMHANDLE               parse_memhandle;
    int                     results_pending;
    int                     result_count;       /* no of stmts that return */
                                                /* or may return a rs */
    void                    *exec_header;
    void                    *current_node;
    void                    *active_rowset;
    LIST                    order_by_list;            
    char                    *order_by_buffer;
    int                     order_by_buffer_length;
    void                    *order_by_sort_handle;
    int                     param_offset;       /* current offset into the */
                                                /* array of parameters */
    int                     row_offset;         /* current offset into the */
                                                /* rowset */
    int                     row_count;
    char                    cursor_name[ MAX_CURSOR_NAME + 1 ];
    int                     cursor_name_set;
    time_t                  statement_timestamp;
    struct Handle_Stmt      *next_stmt;
                            /* 
                             * whatoperation  placed us in SQL_NEED_DATA 
                             * SQL_API_EXECUTE, SQL_API_EXEC_DIRECT
                             * SQL_SET_POS or SQL_BULK_OPERATIONS
                             */
    int                     need_data_entered_from_op; 
                            /* 
                             * what state did we enter SQL_NEED_DATA 
                             * from 
                             */
    int                     need_data_entered_from_state; 
                            /*
                             * contains the current bookmark position
                             */
    Value                   bookmark;
                            /*
                             * a extended fetch is underway, dont use the status array
                             */
    int                     in_extended_fetch;
                            /* 
                             * SetPos is requestion a position, 
                             * dont refresh the buffers
                             */
    int                     dont_refresh; 
                            /*
                             * bit of a slimy hack to cope with the special case of
                             * a select count(*) from table
                             * where table is empty
                             */
    int                     one_time_only;
                            /*
                             * stops the set value evaluator from incrementing the counts
                             */
    int                     dont_increment_counts;
                            /*
                             * Async operation
                             */
    int                     async_enable;
    int                     async_return_code;
    int                     saved_state;
    int                     async_function;
    thread_handle           thread;    
    semaphore_handle        semaphore_in;
    semaphore_handle        semaphore_out;
    int                     cancel_has_been_set;
    int                     timeout_has_been_set;
    int                     update_set_values;      /* flag to make set values updating */
    int                     put_field;              /* current param in putdata */
    int                     repopulate;             /* has bind_param been called */
                                                    /* between prepare and execute ? */
#ifdef WIN32                                    
    int                     async_call;             /* current async call to perform */
    void                    *async_arg;             /* arg to async op */
    void                    (*async_func)(void*);   /* function to call */
#endif
} Handle_Stmt;

typedef struct Desc_Field
{
    int                     type;
    int                     desc_type;
    int                     auto_unique;
    char                    *base_column_name;
    char                    *base_table_name;
    int                     case_sensitive;
    char                    *catalog_name;
    short                   consise_type;
    void                    *data_ptr;
    short                   datetime_interval_code;
    int                     datetime_interval_precision;
    int                     display_size;
    short                   fixed_prec_scale;
    int                     *indicator_ptr;
    char                    *label;
    int                     length;
    char                    literal_prefix[ 64 ];
    char                    literal_suffix[ 64 ];
    char                    local_type_name[ 64 ];
    char                    *name;
    short                   nullable;
    int                     num_prec_radix;
    int                     octet_length;
    int                     *octet_length_ptr;
    short                   parameter_type;
    short                   precision;
    short                   scale;
    char                    *schema_name;
    short                   searchable;
    char                    *table_name;
    char                    type_name[ 64 ];
    short                   unnamed;
    short                   unsig;
    short                   updateable;
    void                    *value;
    int                     value_planted_from_sort;    /* use the value in the     */
                                                        /* value field instead of   */
                                                        /* reevaluating it          */
    int                     column_needed;
    int                     deferred_value;             /* the value has not been evaluated yet */
    void                    *deferred_sr;               /* Exec_SelectRef * */
    void                    *deferred_stmt;             /* Handle_Stmt * */
    void                    *deferred_mh;               /* MEMHANDLE */
    int                     deferred_null;              /* has it been nulled by a join ? */
    int                     deferred_null_set;
} Desc_Field;

typedef struct Handle_Desc
{
    int                     type;
    MEMHANDLE               root_mem_handle;    /* this is a child of the dbc       */
    MEMHANDLE               memhandle;          /* this is a child of the stmt      */
    Handle_Env              *env;
    Handle_Dbc              *dbc;
    Handle_Stmt             *stmt;
    ERRORHANDLE             error_header;
    int                     state;
    int                     populated;
    int                     implicit;
    int                     desc_type;          /* ARD, APD, IRD, IPD */
    short                   alloc_type;
    int                     array_size;
    int                     rowset_size;        /* For extended fetch */
    SQLUSMALLINT            *array_status_ptr;
    SQLUINTEGER             *bind_offset_ptr;
    int                     bind_type;
    short                   count;            
    SQLUINTEGER             *rows_processed_ptr;
    Desc_Field              *fields;
} Handle_Desc;

#endif
