/*
 *  $Id: parsenodes.h,v 1.26 2000/08/14 13:57:04 nick Exp $
 *
 *  $Log: parsenodes.h,v $
 *  Revision 1.26  2000/08/14 13:57:04  nick
 *  Move new fields in value structure
 *
 *  Revision 1.25  2000/08/14 13:08:18  nick
 *  Add join fixes
 *
 *  Revision 1.24  2000/08/14 09:01:00  nick
 *  Add deferred read support
 *
 *  Revision 1.23  2000/08/11 09:19:31  nick
 *  Make C++ frendly
 *
 *  Revision 1.22  2000/07/19 14:15:58  nick
 *  Add extra values to expression for constant propigation
 *
 *  Revision 1.21  2000/07/10 15:18:33  nick
 *  Add 4 part names
 *  Add create/drop user
 *
 *  Revision 1.20  2000/06/19 08:38:54  nick
 *   Add 1.4 support code
 *
 *  Revision 1.19  2000/06/14 12:58:17  nick
 *  Add flag to mark nested query
 *
 *  Revision 1.18  2000/06/12 10:40:29  nick
 *  Add support for subselects in updates
 *
 *  Revision 1.17  2000/06/05 08:19:50  nick
 *  Add passthrough support
 *
 *  Revision 1.16  2000/02/15 12:07:47  nick
 *  Add Procedure support
 *
 *  Revision 1.15  2000/01/12 14:39:50  nick
 *  Add tree based in evaluation
 *
 *  Revision 1.14  1999/12/20 13:09:28  nick
 *  Use int64_t
 *
 *  Revision 1.13  1999/08/13 16:53:06  nick
 *  Add support for new value alloc
 *
 *  Revision 1.12  1999/06/30 13:31:40  nick
 *  Add support for auto increment
 *
 *  Revision 1.11  1999/03/04 22:26:11  nick
 *  Add extra sqi data info to Create Table
 *
 *  Revision 1.10  1999/02/16 17:16:18  nick
 *  Add isnot to exists and quantified
 *
 *  Revision 1.9  1999/02/16 11:39:58  nick
 *  Add support for commit and rollback
 *
 *  Revision 1.8  1999/01/25 16:05:06  nick
 *  Add target_type field in the case structure to fix bug with a case
 *  returing variant types
 *
 *  Revision 1.7  1999/01/04 13:07:18  nick
 *  Add support for expressions in orderby's
 *
 *  Revision 1.6  1998/12/15 10:32:35  nick
 *  Add correlation_table flag to indicate that the correlation
 *  should be found in the tables list not the select list
 *  Add CASE, COALESCE, NULLIF and When structures
 *
 *  Revision 1.5  1998/11/20 13:20:14  nick
 *  Add the binary_length field to a value
 *
 *  Revision 1.4  1998/09/21 14:26:39  nick
 *  Add long buffer support
 *
 *  Revision 1.3  1998/08/18 15:30:17  nick
 *  add extra precisions to the Value structure for intervals
 *  add IntervalValue structure
 *
 *  Revision 1.2  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

#ifndef _PARSENODES
#define _PARSENODES

#ifdef __cplusplus
extern "C" {             /* Assume C declarations for C++   */
#endif  /* __cplusplus */

#if defined ( WIN32 )
#include <windows.h>
#endif

#include <sqltypes.h>
#include <sqlext.h>

#include "nodes.h"
#include "list.h"
#include "tree.h"

typedef void * STATEMENT;

typedef struct Value
{
    int                     type;
    int                     data_type;
    long                    length;
    long                    binary_length;
    long                    char_offset;
    int                     precision;
    int                     scale;
    int                     interval_leading_precision;
    int                     interval_seconds_precision;
    int                     isnull;
    void                    *long_buffer;
    int                     usage;
    void                    *owning_mh;
    int			            deferred_read;
    void                    *deferred_iterator;
    int                     deferred_column;
    void                    *deferred_table_arr;
    int                     sval_in_block;                  /* if this is set then this */
                                                            /* Value str and the memory pointed */
                                                            /* to by sval is allocated by the same */
                                                            /* buffer */
    union
    {    
        int                         ival;
        double                      dval;
        char                        *sval;
        int64_t                     bigint;
        SQL_DATE_STRUCT             date;
        SQL_TIME_STRUCT             time;
        SQL_TIMESTAMP_STRUCT        timestamp;
        SQL_INTERVAL_STRUCT         interval;
        SQL_NUMERIC_STRUCT          numeric;
        int                         *_p_ival4;
        short                       *_p_ival2;
        char                        *_p_ival1;
        double                      *_p_dval;
        int64_t                     *_p_bigint;
        SQL_DATE_STRUCT             *_p_date;
        SQL_TIME_STRUCT             *_p_time;
        SQL_TIMESTAMP_STRUCT        *_p_timestamp;
        SQL_INTERVAL_STRUCT         *_p_interval;
        SQL_NUMERIC_STRUCT          *_p_numeric;
    } x;
} Value;

typedef struct BigValue
{
    Value                   v;
    char                    buffer[ BIG_VALUE_EXTENSION ];
} BigValue;

typedef struct List
{
    int                     type;
    LIST                    list;
} List;

typedef struct Expression
{
    int                     type;
    int                     oper;
    int                     oper_type;
    struct Expression       *lexpr;
    struct Expression       *rexpr;
    Value                   *constant_expr;
    Value                   *left_value_expr;
    Value                   *right_value_expr;
} Expression;

typedef struct Identifier
{
    int                     type;
    char *                  value;
    int                     quoted;
} Identifier;

typedef struct TableName
{
    int                     type;
    Identifier              *link;
    Identifier              *schema;
    Identifier              *catalog;
    Identifier              *name;
} TableName;

typedef struct ColumnName
{
    int                     type;
    Identifier              *link;
    Identifier              *schema;
    Identifier              *catalog;
    Identifier              *table_name;
    Identifier              *column_name;
    void                    *dal_columninfo;
    void                    *correlation_target;
    void                    *correlation_ex;
    int                     correlation_table;
    int                     column_id;
    int                     table_id;           /* to access the ExecSelect info */
    void                    *joined_query;      /* QuerySpecification from the */
                                                /* other part of a join */
    int                     first_in_join;      /* is it the leading part of a join */
    Value                   *value_from_sort;
} ColumnName;

typedef struct AlterTableAdd
{
    int                     type;
    TableName               *table_name;
    List                    *column_definition;
} AlterTableAdd;

typedef struct AlterTableDrop
{
    int                     type;
    TableName               *table_name;
    Identifier              *column_name;
    int                     cascade_or_restrict;
} AlterTableDrop;

typedef struct CreateIndex
{
    int                     type;
    int                    unique;
    TableName               *index_name;
    TableName               *table_name;
    List                    *column_list;
    int                     key_number;
} CreateIndex;

typedef struct CreateTable
{
    int                     type;
    TableName               *table_name;
    List                    *column_list;
    Identifier              *as;
    Identifier              *on;
    Value                   *file_name;
} CreateTable;

typedef struct CreateUser
{
    int                     type;
    Identifier              *user_name;
    Identifier              *password;
    Identifier              *link;
} CreateUser;

typedef struct DropUser
{
    int                     type;
    Identifier              *user_name;
    Identifier              *link;
} DropUser;

typedef struct SelectProlog
{
    int                     type;
    int                     all_distinct;
    List                    *select_list;
    List                    *table_list;
    Expression              *where;
    int                     nested_query;   /* marks as a query created by */
                                            /* a nested join */
} SelectProlog;

typedef struct QuerySpecification
{
    int                     type;
    SelectProlog            *prolog;
    List                    *group;
    Expression              *having;
    void                    *ex_node;       /* for subqueries to find there */
                                            /* way to a exec structure */
} QuerySpecification;

typedef struct QueryEx
{
    int                     type;
    struct QueryEx          *left_union;
    struct QueryEx          *right_union;
    int                     union_all;
    QuerySpecification      *qspec;
} QueryEx;

typedef struct CreateView
{
    int                     type;
    TableName               *table_name;
    List                    *column_list;
    QueryEx                 *query;
    int                     check_option;
} CreateView;

typedef struct DeletePositioned
{
    int                     type;
    TableName               *table_name;
    Identifier              *cursor_name;
} DeletePositioned;

typedef struct DeleteSearched
{
    int                    type;
    TableName            *table_name;
    Expression            *where;
} DeleteSearched;

typedef struct DropIndex
{
    int                    type;
    TableName            *index_name;
} DropIndex;

typedef struct DropTable
{
    int                    type;
    TableName            *table_name;
    int                    cascade_or_restrict;
} DropTable;

typedef struct DropView
{
    int                    type;
    TableName            *view_name;
    int                    cascade_or_restrict;
} DropView;

typedef struct Grant
{
    int                     type;
    List                    *privileges;
    TableName               *table_name;
    List                    *user_list;    
    int                     grant_option;
} Grant;

typedef struct InsertSource
{
    int                     type;
    int                     default_values;
    List                    *column_list;
    QuerySpecification      *qspec;
    List                    *row_list;
} InsertSource;

typedef struct Insert
{
    int                     type;
    TableName               *table_name;
    InsertSource            *insert_source;
} Insert;

typedef struct Revoke
{
    int                     type;
    TableName               *table_name;
    List                    *user_list;
    int                     cascade_or_restrict;
    List                    *privileges;
} Revoke;

typedef struct SelectStmt
{
    int                     type;
    QueryEx                 *query_expression;
    List                    *order_by_list;
} SelectStmt;

typedef struct SelectForUpdate
{
    int                     type;
    SelectProlog            *select_prolog;
    List                    *column_list;
} SelectForUpdate;

typedef struct UpdatePositioned
{
    int                     type;
    TableName               *table_name;
    List                    *set_column_list;
    Identifier              *cursor_name;
} UpdatePositioned;

typedef struct UpdateSearched
{
    int                     type;
    TableName               *table_name;
    List                    *set_column_list;
    Expression              *where;
} UpdateSearched;

typedef struct PrecisionScale
{
    int                     type;
    int                     precision;
    int                     scale;
} PrecisionScale;

typedef struct DataType
{
    int                     type;
    int                     base_type;
    int                     length;
    int                     fpart;
    int                     sign;
    PrecisionScale          *precision_scale;
} DataType;

typedef struct ColumnDefinition
{
    int                     type;
    Identifier              *column_name;
    DataType                *data_type;
    List                    *column_constraint_list;
} ColumnDefinition;

typedef struct TableReference
{
    int                     type;
    TableName               *name;
    Identifier              *correlation_name;
    void                    *joined_table;
    QuerySpecification      *sub_query;
                                                /* needs to be hidden so the name */
                                                /* search can find it, but not */
                                                /* the execute, otherwise we get */
                                                /* recuresion to infinity */
} TableReference;    

typedef struct QualifiedJoin
{
    int                     type;
    TableReference          *table_reference;
    int                     natural;    
    int                     join_type;
    int                     inner;
    TableReference          *joined_table_reference;
    Expression              *search_condition;
    List                    *column_list;
    void                    *exec_joined;
} QualifiedJoin;    

typedef struct JoinType
{
    int                     type;
    int                     join_type;
    int                     inner_outer;
} JoinType;

typedef struct JoinSpecification
{
    int                     type;
    Expression              *search_condition;
    List                    *column_list;
} JoinSpecification;

typedef struct SetColumnIdentifier
{
    int                     type;
    Identifier              *name;
    int                     setnull;
    int                     setdefault;
    Expression              *expression;
    QuerySpecification      *sub_query;
} SetColumnIdentifier;

typedef struct IndexColDef
{
    int                     type;
    Identifier              *column_name;
    int                     direction;
} IndexColDef;

typedef struct TableConstraint
{
    int                     type;
    int                     constraint_type;
    List                    *column_list;
    Expression              *search_condition;
    TableName               *table_name;
    List                    *refered_column_list;
    int                     on_delete;
    int                     key_number;
} TableConstraint;

typedef struct ColumnElement
{
    int                     type;
    ColumnDefinition        *column_definition;
    TableConstraint         *table_constraint;
} ColumnElement;

typedef struct Privilege
{
    int                     type;
    int                     priv;
    List                    *column_name_list;
} Privilege;

typedef struct SelectSublist
{
    int                     type;
    Expression              *expr;
    Identifier              *correlation;
    TableName               *wildcard;
} SelectSublist;

typedef struct SortSpecification
{
    int                     type;
    Expression              *expr;
    int                     explicit_column_number;
    int                     direction;
} SortSpecification;

typedef struct ColConstDef
{
    int                     type;
    int                     constraint_type;
    TableName               *table_name;
    Identifier              *column_name;
    Expression              *search_condition;
    Value                   *default_value;
    Identifier              *as_definition;
    List                    *as_definition_list;
    int                     auto_start;
    int                     auto_inc;
    Value                   *column_string;
    int                     key_number;
} ColConstDef;

typedef struct ColumnSpec
{
    int                     type;
    int                     spec_type;
} ColumnSpec;

typedef struct IntervalField
{
    int                     type;
    int                     duration_type;
    int                     precision;
    PrecisionScale          *precision_scale;
} IntervalField;

typedef struct IntervalQualifier
{
    int                     type;
    IntervalField           *start_field;
    IntervalField           *end_field;
    IntervalField           *single_field;
} IntervalQualifier;

typedef struct CastExpression
{
    int                     type;
    Expression              *expr;
    DataType                *data_type;
} CastExpression;

typedef struct ScalarFunction
{
    int                     type;
    int                     function_type;
    List                    *expr_list;
    Expression              *expr1, *expr2, *expr3;
    void                    *function_info;
} ScalarFunction;

typedef struct TrimExpression
{
    int                     type;
    int                     trim_type;
    Value                   *trim_char;
    Expression              *expression;
} TrimExpression;

typedef struct SetFunction
{
    int                     type;
    int                     function;
    int                     distinct;
    ColumnName              *column_name;
    Expression              *expr;
    void                    *set_value;             /* set to the expr struct */
    int                     consise_type;
} SetFunction;

typedef struct ExistsPredicate
{
    int                     type;
    QuerySpecification      *sub_query;
    int                     isnot;
} ExistsPredicate;

typedef struct LikePredicate
{
    int                     type;
    Expression              *expr;
    int                     isnot;
    Value                   *pattern_value;
    Value                   *escape_char;
    Value                   *regexp;
} LikePredicate;

typedef struct NullPredicate
{
    int                     type;
    ColumnName              *column_name;
    int                     isnot;
} NullPredicate;

typedef struct InPredicate
{
    int                     type;
    Expression              *expr;
    int                     notin;
    QuerySpecification      *sub_query;
    TREEHANDLE              in_tree;
    int                     tree_full;
    List                    *in_list;
} InPredicate;

typedef struct QuantifiedPredicate
{
    int                     type;
    Expression              *expr;
#ifdef __cplusplus
    int                     _operator;          // nasty hack to get by C++ 
#else
    int                     operator;
#endif
    int                     all_any_some;
    int                     isnot;
    QuerySpecification      *sub_query;
} QuantifiedPredicate;

typedef struct Parameter
{
    int                     type;
    int                     id;
    Expression              *partner;
    void                    *column_info;
    int                     proc_arg;
} Parameter;

typedef struct Procedure
{
    int                     type;
    TableName               *name;
    List                    *param;
    int                     isvoid;
    Parameter               *return_val;
} Procedure;

typedef struct IntervalValue
{
    int                     type;
    int                     prespace;
    int                     i1;
    int                     i2;
    Value                   *seconds;
    int                     val_count;
    int                     hasspace;
    int                     hasdash;
} IntervalValue;

typedef struct CaseExpression
{
    int                     type;
    Expression              *operand;
    List                    *when_list;
    Expression              *else_clause;
    int                     target_sql_type;
} CaseExpression;

typedef struct NullIfExpression
{
    int                     type;
    Expression              *v1;
    Expression              *v2;
} NullIfExpression;

typedef struct CoalesceExpression
{
    int                     type;
    List                    *expression_list;
} CoalesceExpression;

typedef struct WhenClause
{
    int                     type;
    Expression              *operand;
    Expression              *result;
} WhenClause;

typedef struct Commit
{
    int                     type;
} Commit;

typedef struct Rollback
{
    int                     type;
} Rollback;

typedef struct DataDefinition
{
    int                     type;
    Identifier              *value;
    Expression              *expr;
} DataDefinition;

#ifdef __cplusplus
}                                    /* End of extern "C" { */
#endif  /* __cplusplus */
#endif
