/*
 *  $Id: threads.h,v 1.9 2000/11/21 15:58:53 martin Exp $
 *
 *  $Log: threads.h,v $
 *  Revision 1.9  2000/11/21 15:58:53  martin
 *  Add support for GNU PTH.
 *
 *  Revision 1.8  2000/02/25 09:30:17  nick
 *  Fix incorrect #include
 *
 *  Revision 1.7  2000/02/24 15:41:02  nick
 *  Add initial Solaris support
 *
 *  Revision 1.6  1999/12/20 12:06:00  nick
 *  Alter thread_init to odbc_thread_init
 *
 *  Revision 1.5  1999/12/20 11:57:24  nick
 *  Alter name thread_create to avoid clash with AIX threads
 *
 *  Revision 1.4  1999/12/13 14:08:41  nick
 *  Add schema mutex
 *
 *  Revision 1.3  1999/09/22 11:35:04  nick
 *  Add support for windows threads
 *
 *  Revision 1.2  1999/06/30 13:32:29  nick
 *  Add support for the startup functions
 *
 *  Revision 1.1  1998/12/03 17:25:28  nick
 *  Initial revision
 *
 *
 */

#ifndef __THREADS
#define __THREADS

#ifdef _WITH_THREADS
#ifdef __unix
#ifdef HAVE_LIBPTHREADS
#include <pthread.h>

typedef pthread_t thread_handle;

#elif HAVE_LIBTHREAD
#include <thread.h>

typedef thread_t thread_handle;
#elif WITH_GNUPTH
#include <pth.h>

typedef pth_t thread_handle;
#endif

typedef int semaphore_handle;
int mutex_entry( int );
int mutex_exit( int );

#endif  /* __unix */

#ifdef WIN32
#include <process.h>

typedef unsigned thread_handle;
typedef unsigned semaphore_handle;

int mutex_entry( int );
int mutex_exit( int );

#endif  /* WIN32 */
#else   /* no threads */

typedef int thread_handle;
typedef int semaphore_handle;

#define mutex_entry( x )
#define mutex_exit( x )

#endif

#define MUTEX_PARSER        0
#define MUTEX_MEMORY        1
#define MUTEX_DATAIO        2
#define MUTEX_STATES        3
#define MUTEX_DBC           4
#define MUTEX_SCHEMA        5
#define MUTEX_THREAD        6

int can_connect_async( void );
int can_statement_async( void );
int odbc_thread_create( thread_handle *, void (*)(void*), void *, 
        semaphore_handle *sem_in, semaphore_handle *sem_out );
int thread_wait_on_semaphore( semaphore_handle sem, int msec );
void thread_clear_semaphore( semaphore_handle sem );
void thread_close_semaphore( semaphore_handle sem );
int thread_cleanup( thread_handle );
void odbc_thread_init( int );
void thread_term( int );

#endif
