/*
 *  $Id: validate.h,v 1.17 2000/08/18 11:54:31 nick Exp $
 *
 *  $Log: validate.h,v $
 *  Revision 1.17  2000/08/18 11:54:31  nick
 *  Remove mark_needed
 *
 *  Revision 1.16  2000/08/17 12:48:48  nick
 *  Add table_id to mark_needed
 *
 *  Revision 1.15  2000/08/11 09:19:50  nick
 *  Add mark_needed
 *
 *  Revision 1.14  2000/07/10 15:19:31  nick
 *  Add 4 part names
 *
 *  Revision 1.13  2000/06/19 08:39:23  nick
 *   Add 1.4 support code
 *
 *  Revision 1.12  2000/02/15 12:07:47  nick
 *  Add Procedure support
 *
 *  Revision 1.11  1999/03/17 17:19:52  nick
 *  add check_names function
 *
 *  Revision 1.10  1999/02/16 11:42:58  nick
 *  remove global defs for validate_column_element and validate_column_definition
 *  add validate_commit and validate_rollback
 *
 *  Revision 1.9  1999/01/25 16:06:01  nick
 *  Add extra fields to support correlated queries and the new name extraction
 *
 *  Revision 1.8  1998/12/15 10:34:18  nick
 *  Add correlated_table to callback args
 *
 *  Revision 1.7  1998/11/18 11:50:07  nick
 *  Add extra param to setup_types to indicate if odbc 2 or 3 is needed
 *
 *  Revision 1.6  1998/11/10 13:35:14  nick
 *  Add VIEW support
 *
 *  Revision 1.5  1998/09/21 14:27:04  nick
 *  Replace get_base_type with type_base
 *
 *  Revision 1.4  1998/08/18 15:31:11  nick
 *  Joins finished
 *
 *  Revision 1.3  1998/08/12 08:52:49  nick
 *  Checkpoint
 *
 *  Revision 1.2  1998/07/13 08:14:32  nick
 *  remove virtual join support.
 *
 *  Revision 1.1  1998/05/08 08:54:56  nick
 *  Initial revision
 *
 *
 */

#ifndef _VALIDATE
#define _VALIDATE

struct validate_arg
{
    Handle_Stmt         *stmt;
    jmp_buf             env;
    int                 retval;
    void                *exnode;
    void                *joinnode;                  /* used to carry the Exec_Joined about */
    int                 in_select_list;
    int                 in_having;
    int                 set_functions_in_query;
    int                 exclude_count;              /* number of columns to exclude from NATURAL */
                                                    /* joins */
    int                 *blank_arr;
};

struct enum_name_struct 
{
    Identifier          *correlation_name;
    DALCOLUMNINFO       *dal_info;
    void                (*found_one)( struct enum_name_struct * ens );
    void                *data;
    Exec_Select         *exec_select;
    Exec_Select         *exec_joined;
    TableReference      *table_ref;
    Exec_TableRef       *exec_table_ref;
    Exec_SelectRef      *correlation_target;
    Exec_Select         *sub_select_exec;
    Exec_Select         *correlation_ex;
    SelectSublist       *esl;
    int                 column_id;
    int                 table_id;
    int                 correlated_subq;
};

struct validate_cb_struct
{
    ColumnName          *c;
    int                 *found_count;
    DALCOLUMNINFO       **match;
    int                 *match_table;
    int                 *match_column;
    Exec_Select         **correlation_ex;
    QuerySpecification  *joined_query;
    Exec_SelectRef      **correlation_target;
    int                 correlated_subq;
    int                 correlated_table;
    struct validate_arg *va;
};

int compare_names( void *, void * );
char * extract_schema( TableName * tn );
char * extract_name( TableName * tn );
char * extract_catalog( TableName * tn );
char * extract_link( TableName * tn );
char * create_name( TableName * tn );
void check_names( TableName * tn, struct validate_arg * va );

int extract_type_from_node( void * node, struct validate_arg * va );
void validate_query_specification( QuerySpecification * p, struct validate_arg * va );
void validate_query_expression( QueryEx * p, struct validate_arg * va );
int type_base( int );
void extract_extended_type( void * node, struct validate_arg * va, 
    ColumnDataType * ex_type );
void enumerate_names( void * node, struct enum_name_struct *ens );
void setup_types( int TargetType, Desc_Field * field, int odbc_version );

void validate_general_error( struct validate_arg * va, char * txt );
void validate_distinct_error( struct validate_arg * va, char * sqlstate, char * txt );
void validate_select_stmt( SelectStmt * p, struct validate_arg * va );
void validate_select_upd_stmt( SelectForUpdate * p, struct validate_arg * va );
void validate_create_table( CreateTable * p, struct validate_arg * va );
void validate_drop_table( DropTable * p, struct validate_arg * va );
void validate_create_index( CreateIndex * p, struct validate_arg * va );
void validate_drop_index( DropIndex * p, struct validate_arg * va );
void validate_insert( Insert * p, struct validate_arg * va );
void validate_delete( DeleteSearched * p, struct validate_arg * va );
void validate_update( UpdateSearched * p, struct validate_arg * va );
void validate_delete_pos( DeletePositioned * p, struct validate_arg * va );
void validate_update_pos( UpdatePositioned * p, struct validate_arg * va );
void validate_grant( Grant * p, struct validate_arg * va );
void validate_revoke( Revoke * p, struct validate_arg * va );
void validate_alter_drop( AlterTableDrop * p, struct validate_arg * va );
void validate_alter_add( AlterTableAdd * ata, struct validate_arg * va );
void validate_create_view( CreateView * p, struct validate_arg * va );
void validate_drop_view( DropView * p, struct validate_arg * va );
void validate_commit( Commit * p, struct validate_arg * va );
void validate_rollback( Rollback * p, struct validate_arg * va );
void validate_procedure( Procedure * p, struct validate_arg * va );
void validate_create_user( CreateUser * p, struct validate_arg * va );
void validate_drop_user( DropUser * p, struct validate_arg * va );

void validate_sub_query( struct validate_arg * va, QuerySpecification * qs, 
    int add_to_list, Identifier *correlation_name );
void extract_or_func( void * ve, void * arg );

#endif
